@@ -1,22 +1,22 @@
-use vortex::compute::{slice, SliceFn};
-use vortex::{Array, ArrayDType, IntoArray};
+use vortex_array::compute::{slice, SliceFn};
+use vortex_array::{ArrayDType, ArrayData, IntoArrayData};
 use vortex_error::VortexResult;
 
-use crate::ALPRDArray;
+use crate::{ALPRDArray, ALPRDEncoding};
 
-impl SliceFn for ALPRDArray {
-    fn slice(&self, start: usize, stop: usize) -> VortexResult<Array> {
-        let left_parts_exceptions = self
+impl SliceFn<ALPRDArray> for ALPRDEncoding {
+    fn slice(&self, array: &ALPRDArray, start: usize, stop: usize) -> VortexResult<ArrayData> {
+        let left_parts_exceptions = array
             .left_parts_exceptions()
             .map(|array| slice(&array, start, stop))
             .transpose()?;
 
         Ok(ALPRDArray::try_new(
-            self.dtype().clone(),
-            slice(self.left_parts(), start, stop)?,
-            self.left_parts_dict(),
-            slice(self.right_parts(), start, stop)?,
-            self.right_bit_width(),
+            array.dtype().clone(),
+            slice(array.left_parts(), start, stop)?,
+            array.left_parts_dict(),
+            slice(array.right_parts(), start, stop)?,
+            array.right_bit_width(),
             left_parts_exceptions,
         )?
         .into_array())
@@ -26,9 +26,9 @@ impl SliceFn for ALPRDArray {
 #[cfg(test)]
 mod test {
     use rstest::rstest;
-    use vortex::array::PrimitiveArray;
-    use vortex::compute::slice;
-    use vortex::IntoArrayVariant;
+    use vortex_array::array::PrimitiveArray;
+    use vortex_array::compute::slice;
+    use vortex_array::IntoArrayVariant;
 
     use crate::{ALPRDFloat, RDEncoder};
 
