@@ -1,16 +1,20 @@
-use std::fmt::Debug;
+use std::fmt::{Debug, Display};
 
 use serde::{Deserialize, Serialize};
-use vortex::array::StructArray;
-use vortex::compute::unary::try_cast;
-use vortex::encoding::ids;
-use vortex::stats::{ArrayStatisticsCompute, StatsSet};
-use vortex::validity::{ArrayValidity, LogicalValidity, Validity};
-use vortex::variants::{ArrayVariants, ExtensionArrayTrait};
-use vortex::visitor::{AcceptArrayVisitor, ArrayVisitor};
-use vortex::{impl_encoding, Array, ArrayDType, ArrayTrait, Canonical, IntoArray, IntoCanonical};
+use vortex_array::array::StructArray;
+use vortex_array::compute::unary::try_cast;
+use vortex_array::encoding::ids;
+use vortex_array::stats::{Stat, StatisticsVTable, StatsSet};
+use vortex_array::validity::{LogicalValidity, Validity, ValidityVTable};
+use vortex_array::variants::{ArrayVariants, ExtensionArrayTrait};
+use vortex_array::visitor::{ArrayVisitor, VisitorVTable};
+use vortex_array::{
+    impl_encoding, ArrayDType, ArrayData, ArrayLen, ArrayTrait, Canonical, IntoArrayData,
+    IntoCanonical,
+};
 use vortex_dtype::{DType, PType};
 use vortex_error::{vortex_bail, VortexExpect as _, VortexResult, VortexUnwrap};
+use vortex_scalar::Scalar;
 
 use crate::compute::decode_to_temporal;
 
@@ -25,12 +29,18 @@ pub struct DateTimePartsMetadata {
     subseconds_ptype: PType,
 }
 
+impl Display for DateTimePartsMetadata {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        Debug::fmt(self, f)
+    }
+}
+
 impl DateTimePartsArray {
     pub fn try_new(
         dtype: DType,
-        days: Array,
-        seconds: Array,
-        subsecond: Array,
+        days: ArrayData,
+        seconds: ArrayData,
+        subsecond: ArrayData,
     ) -> VortexResult<Self> {
         if !days.dtype().is_int() || (dtype.is_nullable() != days.dtype().is_nullable()) {
             vortex_bail!(
@@ -67,11 +77,11 @@ impl DateTimePartsArray {
             length,
             metadata,
             [days, seconds, subsecond].into(),
-            StatsSet::new(),
+            StatsSet::default(),
         )
     }
 
-    pub fn days(&self) -> Array {
+    pub fn days(&self) -> ArrayData {
         self.as_ref()
             .child(
                 0,
@@ -81,13 +91,13 @@ impl DateTimePartsArray {
             .vortex_expect("DatetimePartsArray missing days array")
     }
 
-    pub fn seconds(&self) -> Array {
+    pub fn seconds(&self) -> ArrayData {
         self.as_ref()
             .child(1, &self.metadata().seconds_ptype.into(), self.len())
             .vortex_expect("DatetimePartsArray missing seconds array")
     }
 
-    pub fn subsecond(&self) -> Array {
+    pub fn subsecond(&self) -> ArrayData {
         self.as_ref()
             .child(2, &self.metadata().subseconds_ptype.into(), self.len())
             .vortex_expect("DatetimePartsArray missing subsecond array")
@@ -113,7 +123,7 @@ impl ArrayVariants for DateTimePartsArray {
 }
 
 impl ExtensionArrayTrait for DateTimePartsArray {
-    fn storage_array(&self) -> Array {
+    fn storage_data(&self) -> ArrayData {
         // FIXME(ngates): this needs to be a tuple array so we can implement Compare
         // we don't want to write validity twice, so we pull it up to the top
         let days = try_cast(self.days(), &self.days().dtype().as_nonnullable()).vortex_unwrap();
@@ -134,22 +144,39 @@ impl IntoCanonical for DateTimePartsArray {
     }
 }
 
-impl ArrayValidity for DateTimePartsArray {
-    fn is_valid(&self, index: usize) -> bool {
-        self.validity().is_valid(index)
+impl ValidityVTable<DateTimePartsArray> for DateTimePartsEncoding {
+    fn is_valid(&self, array: &DateTimePartsArray, index: usize) -> bool {
+        array.validity().is_valid(index)
     }
 
-    fn logical_validity(&self) -> LogicalValidity {
-        self.validity().to_logical(self.len())
+    fn logical_validity(&self, array: &DateTimePartsArray) -> LogicalValidity {
+        array.validity().to_logical(array.len())
     }
 }
 
-impl AcceptArrayVisitor for DateTimePartsArray {
-    fn accept(&self, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
-        visitor.visit_child("days", &self.days())?;
-        visitor.visit_child("seconds", &self.seconds())?;
-        visitor.visit_child("subsecond", &self.subsecond())
+impl VisitorVTable<DateTimePartsArray> for DateTimePartsEncoding {
+    fn accept(
+        &self,
+        array: &DateTimePartsArray,
+        visitor: &mut dyn ArrayVisitor,
+    ) -> VortexResult<()> {
+        visitor.visit_child("days", &array.days())?;
+        visitor.visit_child("seconds", &array.seconds())?;
+        visitor.visit_child("subsecond", &array.subsecond())
     }
 }
 
-impl ArrayStatisticsCompute for DateTimePartsArray {}
+impl StatisticsVTable<DateTimePartsArray> for DateTimePartsEncoding {
+    fn compute_statistics(&self, array: &DateTimePartsArray, stat: Stat) -> VortexResult<StatsSet> {
+        let maybe_stat = match stat {
+            Stat::NullCount => Some(Scalar::from(array.validity().null_count(array.len())?)),
+            _ => None,
+        };
+
+        let mut stats = StatsSet::default();
+        if let Some(value) = maybe_stat {
+            stats.set(stat, value);
+        }
+        Ok(stats)
+    }
+}
