@@ -1,14 +1,14 @@
-use vortex::array::{PrimitiveArray, TemporalArray};
-use vortex::compute::unary::try_cast;
-use vortex::{Array, ArrayDType as _, IntoArray, IntoArrayVariant};
+use vortex_array::array::{PrimitiveArray, TemporalArray};
+use vortex_array::compute::unary::try_cast;
+use vortex_array::{ArrayDType as _, ArrayData, ArrayLen, IntoArrayData, IntoArrayVariant};
 use vortex_datetime_dtype::TimeUnit;
 use vortex_dtype::{DType, PType};
 use vortex_error::{vortex_bail, VortexResult};
 
 pub struct TemporalParts {
-    pub days: Array,
-    pub seconds: Array,
-    pub subseconds: Array,
+    pub days: ArrayData,
+    pub seconds: ArrayData,
+    pub subseconds: ArrayData,
 }
 
 /// Compress a `TemporalArray` into day, second, and subsecond components.
@@ -17,14 +17,14 @@ pub struct TemporalParts {
 /// cascading compression.
 pub fn split_temporal(array: TemporalArray) -> VortexResult<TemporalParts> {
     let temporal_values = array.temporal_values().into_primitive()?;
-    let validity = temporal_values.validity().clone();
+    let validity = temporal_values.validity();
 
     // After this operation, timestamps will be non-nullable PrimitiveArray<i64>
     let timestamps = try_cast(
         &temporal_values,
         &DType::Primitive(PType::I64, temporal_values.dtype().nullability()),
     )?
-    .as_primitive();
+    .into_primitive()?;
 
     let divisor = match array.temporal_metadata().time_unit() {
         TimeUnit::Ns => 1_000_000_000,
