@@ -1,72 +1,80 @@
 use itertools::Itertools as _;
-use vortex::array::{PrimitiveArray, TemporalArray};
-use vortex::compute::unary::{scalar_at, ScalarAtFn};
-use vortex::compute::{slice, take, ArrayCompute, SliceFn, TakeFn};
-use vortex::validity::ArrayValidity;
-use vortex::{Array, ArrayDType, IntoArray, IntoArrayVariant};
+use vortex_array::array::{PrimitiveArray, TemporalArray};
+use vortex_array::compute::unary::{scalar_at, ScalarAtFn};
+use vortex_array::compute::{slice, take, ComputeVTable, SliceFn, TakeFn, TakeOptions};
+use vortex_array::validity::ArrayValidity;
+use vortex_array::{ArrayDType, ArrayData, IntoArrayData, IntoArrayVariant};
 use vortex_datetime_dtype::{TemporalMetadata, TimeUnit};
-use vortex_dtype::{DType, PType};
-use vortex_error::{vortex_bail, VortexResult, VortexUnwrap as _};
+use vortex_dtype::DType;
+use vortex_error::{vortex_bail, VortexResult};
 use vortex_scalar::Scalar;
 
-use crate::DateTimePartsArray;
+use crate::{DateTimePartsArray, DateTimePartsEncoding};
 
-impl ArrayCompute for DateTimePartsArray {
-    fn scalar_at(&self) -> Option<&dyn ScalarAtFn> {
+impl ComputeVTable for DateTimePartsEncoding {
+    fn scalar_at_fn(&self) -> Option<&dyn ScalarAtFn<ArrayData>> {
         Some(self)
     }
 
-    fn slice(&self) -> Option<&dyn SliceFn> {
+    fn slice_fn(&self) -> Option<&dyn SliceFn<ArrayData>> {
         Some(self)
     }
 
-    fn take(&self) -> Option<&dyn TakeFn> {
+    fn take_fn(&self) -> Option<&dyn TakeFn<ArrayData>> {
         Some(self)
     }
 }
 
-impl TakeFn for DateTimePartsArray {
-    fn take(&self, indices: &Array) -> VortexResult<Array> {
-        Ok(Self::try_new(
-            self.dtype().clone(),
-            take(self.days(), indices)?,
-            take(self.seconds(), indices)?,
-            take(self.subsecond(), indices)?,
+impl TakeFn<DateTimePartsArray> for DateTimePartsEncoding {
+    fn take(
+        &self,
+        array: &DateTimePartsArray,
+        indices: &ArrayData,
+        options: TakeOptions,
+    ) -> VortexResult<ArrayData> {
+        Ok(DateTimePartsArray::try_new(
+            array.dtype().clone(),
+            take(array.days(), indices, options)?,
+            take(array.seconds(), indices, options)?,
+            take(array.subsecond(), indices, options)?,
         )?
         .into_array())
     }
 }
 
-impl SliceFn for DateTimePartsArray {
-    fn slice(&self, start: usize, stop: usize) -> VortexResult<Array> {
-        Ok(Self::try_new(
-            self.dtype().clone(),
-            slice(self.days(), start, stop)?,
-            slice(self.seconds(), start, stop)?,
-            slice(self.subsecond(), start, stop)?,
+impl SliceFn<DateTimePartsArray> for DateTimePartsEncoding {
+    fn slice(
+        &self,
+        array: &DateTimePartsArray,
+        start: usize,
+        stop: usize,
+    ) -> VortexResult<ArrayData> {
+        Ok(DateTimePartsArray::try_new(
+            array.dtype().clone(),
+            slice(array.days(), start, stop)?,
+            slice(array.seconds(), start, stop)?,
+            slice(array.subsecond(), start, stop)?,
         )?
         .into_array())
     }
 }
 
-impl ScalarAtFn for DateTimePartsArray {
-    fn scalar_at(&self, index: usize) -> VortexResult<Scalar> {
-        let DType::Extension(ext, nullability) = self.dtype().clone() else {
+impl ScalarAtFn<DateTimePartsArray> for DateTimePartsEncoding {
+    fn scalar_at(&self, array: &DateTimePartsArray, index: usize) -> VortexResult<Scalar> {
+        let DType::Extension(ext) = array.dtype().clone() else {
             vortex_bail!(
                 "DateTimePartsArray must have extension dtype, found {}",
-                self.dtype()
+                array.dtype()
             );
         };
 
-        let TemporalMetadata::Timestamp(time_unit, _) = TemporalMetadata::try_from(&ext)? else {
+        let TemporalMetadata::Timestamp(time_unit, _) = TemporalMetadata::try_from(ext.as_ref())?
+        else {
             vortex_bail!("Metadata must be Timestamp, found {}", ext.id());
         };
 
-        if !self.is_valid(index) {
-            return Ok(Scalar::extension(
-                ext,
-                Scalar::null(DType::Primitive(PType::I64, nullability)),
-            ));
+        if !array.is_valid(index) {
+            return Ok(Scalar::null(DType::Extension(ext)));
         }
 
         let divisor = match time_unit {
@@ -77,32 +85,25 @@ impl ScalarAtFn for DateTimePartsArray {
             TimeUnit::D => vortex_bail!("Invalid time unit D"),
         };
 
-        let days: i64 = scalar_at(self.days(), index)?.try_into()?;
-        let seconds: i64 = scalar_at(self.seconds(), index)?.try_into()?;
-        let subseconds: i64 = scalar_at(self.subsecond(), index)?.try_into()?;
+        let days: i64 = scalar_at(array.days(), index)?.try_into()?;
+        let seconds: i64 = scalar_at(array.seconds(), index)?.try_into()?;
+        let subseconds: i64 = scalar_at(array.subsecond(), index)?.try_into()?;
 
         let scalar = days * 86_400 * divisor + seconds * divisor + subseconds;
 
-        Ok(Scalar::extension(
-            ext,
-            Scalar::primitive(scalar, nullability),
-        ))
-    }
-
-    fn scalar_at_unchecked(&self, index: usize) -> Scalar {
-        <Self as ScalarAtFn>::scalar_at(self, index).vortex_unwrap()
+        Ok(Scalar::extension(ext, Scalar::from(scalar)))
     }
 }
 
-/// Decode an [Array] into a [TemporalArray].
+/// Decode an [ArrayData] into a [TemporalArray].
 ///
 /// Enforces that the passed array is actually a [DateTimePartsArray] with proper metadata.
 pub fn decode_to_temporal(array: &DateTimePartsArray) -> VortexResult<TemporalArray> {
-    let DType::Extension(ext, _) = array.dtype().clone() else {
+    let DType::Extension(ext) = array.dtype().clone() else {
         vortex_bail!(ComputeError: "expected dtype to be DType::Extension variant")
     };
 
-    let Ok(temporal_metadata) = TemporalMetadata::try_from(&ext) else {
+    let Ok(temporal_metadata) = TemporalMetadata::try_from(ext.as_ref()) else {
         vortex_bail!(ComputeError: "must decode TemporalMetadata from extension metadata");
     };
 
@@ -127,7 +128,7 @@ pub fn decode_to_temporal(array: &DateTimePartsArray) -> VortexResult<TemporalAr
         .collect::<Vec<_>>();
 
     Ok(TemporalArray::new_timestamp(
-        PrimitiveArray::from_vec(values, array.validity().clone()).into_array(),
+        PrimitiveArray::from_vec(values, array.validity()).into_array(),
         temporal_metadata.time_unit(),
         temporal_metadata.time_zone().map(ToString::to_string),
     ))
@@ -135,9 +136,9 @@ pub fn decode_to_temporal(array: &DateTimePartsArray) -> VortexResult<TemporalAr
 
 #[cfg(test)]
 mod test {
-    use vortex::array::{PrimitiveArray, TemporalArray};
-    use vortex::validity::Validity;
-    use vortex::{IntoArray, IntoArrayVariant};
+    use vortex_array::array::{PrimitiveArray, TemporalArray};
+    use vortex_array::validity::Validity;
+    use vortex_array::{IntoArrayVariant, ToArrayData};
     use vortex_datetime_dtype::TimeUnit;
     use vortex_dtype::DType;
 
@@ -155,7 +156,7 @@ mod test {
         do_roundtrip_test(&raw_values, Validity::NonNullable);
         do_roundtrip_test(&raw_values, Validity::AllValid);
         do_roundtrip_test(&raw_values, Validity::AllInvalid);
-        do_roundtrip_test(&raw_values, Validity::from(vec![true, false, true]));
+        do_roundtrip_test(&raw_values, Validity::from_iter([true, false, true]));
     }
 
     fn do_roundtrip_test(raw_values: &[i64], validity: Validity) {
@@ -163,7 +164,7 @@ mod test {
         assert_eq!(raw_millis.validity(), validity);
 
         let temporal_array = TemporalArray::new_timestamp(
-            raw_millis.clone().into_array(),
+            raw_millis.to_array(),
             TimeUnit::Ms,
             Some("UTC".to_string()),
         );
@@ -181,13 +182,19 @@ mod test {
             seconds,
             subseconds,
         } = split_temporal(temporal_array.clone()).unwrap();
-        assert_eq!(days.as_primitive().validity(), validity);
-        assert_eq!(seconds.as_primitive().validity(), Validity::NonNullable);
-        assert_eq!(subseconds.as_primitive().validity(), Validity::NonNullable);
+        assert_eq!(days.clone().into_primitive().unwrap().validity(), validity);
+        assert_eq!(
+            seconds.clone().into_primitive().unwrap().validity(),
+            Validity::NonNullable
+        );
+        assert_eq!(
+            subseconds.clone().into_primitive().unwrap().validity(),
+            Validity::NonNullable
+        );
         assert_eq!(validity, raw_millis.validity());
 
         let date_times = DateTimePartsArray::try_new(
-            DType::Extension(temporal_array.ext_dtype().clone(), validity.nullability()),
+            DType::Extension(temporal_array.ext_dtype()),
             days,
             seconds,
             subseconds,
