@@ -5,72 +5,82 @@ use std::cmp::Ordering::Greater;
 use fastlanes::BitPacking;
 use itertools::Itertools;
 use num_traits::AsPrimitive;
-use vortex::array::SparseArray;
-use vortex::compute::{
-    search_sorted_u64, IndexOrd, Len, SearchResult, SearchSorted, SearchSortedFn, SearchSortedSide,
+use vortex_array::array::SparseArray;
+use vortex_array::compute::{
+    search_sorted_usize, IndexOrd, Len, SearchResult, SearchSorted, SearchSortedFn,
+    SearchSortedSide,
 };
-use vortex::validity::Validity;
-use vortex::ArrayDType;
+use vortex_array::validity::Validity;
+use vortex_array::variants::PrimitiveArrayTrait;
+use vortex_array::{ArrayDType, ArrayLen};
 use vortex_dtype::{match_each_unsigned_integer_ptype, NativePType};
 use vortex_error::{VortexError, VortexExpect as _, VortexResult};
 use vortex_scalar::Scalar;
 
-use crate::{unpack_single_primitive, BitPackedArray};
+use crate::{unpack_single_primitive, BitPackedArray, BitPackedEncoding};
 
-impl SearchSortedFn for BitPackedArray {
-    fn search_sorted(&self, value: &Scalar, side: SearchSortedSide) -> VortexResult<SearchResult> {
-        match_each_unsigned_integer_ptype!(self.ptype(), |$P| {
-            search_sorted_typed::<$P>(self, value, side)
+impl SearchSortedFn<BitPackedArray> for BitPackedEncoding {
+    fn search_sorted(
+        &self,
+        array: &BitPackedArray,
+        value: &Scalar,
+        side: SearchSortedSide,
+    ) -> VortexResult<SearchResult> {
+        match_each_unsigned_integer_ptype!(array.ptype(), |$P| {
+            search_sorted_typed::<$P>(array, value, side)
         })
     }
 
-    fn search_sorted_u64(&self, value: u64, side: SearchSortedSide) -> VortexResult<SearchResult> {
-        match_each_unsigned_integer_ptype!(self.ptype(), |$P| {
+    fn search_sorted_usize(
+        &self,
+        array: &BitPackedArray,
+        value: usize,
+        side: SearchSortedSide,
+    ) -> VortexResult<SearchResult> {
+        match_each_unsigned_integer_ptype!(array.ptype(), |$P| {
             // NOTE: conversion may truncate silently.
-            if let Some(pvalue) = num_traits::cast::<u64, $P>(value) {
-                search_sorted_native(self, pvalue, side)
+            if let Some(pvalue) = num_traits::cast::<usize, $P>(value) {
+                search_sorted_native(array, pvalue, side)
             } else {
                 // provided u64 is too large to fit in the provided PType, value must be off
                 // the right end of the array.
-                Ok(SearchResult::NotFound(self.len()))
+                Ok(SearchResult::NotFound(array.len()))
             }
         })
     }
 
     fn search_sorted_many(
         &self,
+        array: &BitPackedArray,
         values: &[Scalar],
-        sides: &[SearchSortedSide],
+        side: SearchSortedSide,
     ) -> VortexResult<Vec<SearchResult>> {
-        match_each_unsigned_integer_ptype!(self.ptype(), |$P| {
-            let searcher = BitPackedSearch::<'_, $P>::new(self);
+        match_each_unsigned_integer_ptype!(array.ptype(), |$P| {
+            let searcher = BitPackedSearch::<'_, $P>::new(array);
 
             values
                 .iter()
-                .zip(sides.iter().copied())
-                .map(|(value, side)| {
+                .map(|value| {
                     // Unwrap to native value
-                    let unwrapped_value: $P = value.cast(self.dtype())?.try_into()?;
-
+                    let unwrapped_value: $P = value.cast(array.dtype())?.try_into()?;
                     Ok(searcher.search_sorted(&unwrapped_value, side))
                 })
                 .try_collect()
         })
     }
 
-    fn search_sorted_u64_many(
+    fn search_sorted_usize_many(
         &self,
-        values: &[u64],
-        sides: &[SearchSortedSide],
+        array: &BitPackedArray,
+        values: &[usize],
+        side: SearchSortedSide,
     ) -> VortexResult<Vec<SearchResult>> {
-        match_each_unsigned_integer_ptype!(self.ptype(), |$P| {
-            let searcher = BitPackedSearch::<'_, $P>::new(self);
+        match_each_unsigned_integer_ptype!(array.ptype(), |$P| {
+            let searcher = BitPackedSearch::<'_, $P>::new(array);
 
             values
                 .iter()
-                .copied()
-                .zip(sides.iter().copied())
-                .map(|(value, side)| {
+                .map(|&value| {
                     // NOTE: truncating cast
                     let cast_value: $P = value as $P;
                     Ok(searcher.search_sorted(&cast_value, side))
@@ -110,7 +120,7 @@ where
         // max packed value just search the patches
         let usize_value: usize = value.as_();
         if usize_value > array.max_packed_value() {
-            search_sorted_u64(&patches_array, value.as_(), side)
+            search_sorted_usize(&patches_array, value.as_(), side)
         } else {
             Ok(BitPackedSearch::<'_, T>::new(array).search_sorted(&value, side))
         }
@@ -125,9 +135,9 @@ struct BitPackedSearch<'a, T> {
     // NOTE: caching this here is important for performance, as each call to `maybe_null_slice`
     //  invokes a call to DType <> PType conversion
     packed_maybe_null_slice: &'a [T],
-    offset: usize,
+    offset: u16,
     length: usize,
-    bit_width: usize,
+    bit_width: u8,
     first_invalid_idx: usize,
 }
 
@@ -146,7 +156,11 @@ impl<'a, T: BitPacking + NativePType> BitPackedSearch<'a, T> {
             Validity::AllInvalid => 0,
             Validity::Array(varray) => {
                 // In sorted order, nulls come after all the non-null values.
-                varray.with_dyn(|a| a.as_bool_array_unchecked().true_count())
+                varray.with_dyn(|a| {
+                    a.statistics()
+                        .compute_true_count()
+                        .vortex_expect("Failed to compute true count")
+                })
             }
         };
 
@@ -172,11 +186,11 @@ impl<T: BitPacking + NativePType> IndexOrd<T> for BitPackedSearch<'_, T> {
         let val: T = unsafe {
             unpack_single_primitive(
                 self.packed_maybe_null_slice,
-                self.bit_width,
-                idx + self.offset,
+                self.bit_width as usize,
+                idx + self.offset as usize,
             )
         };
-        Some(val.compare(*elem))
+        Some(val.total_compare(*elem))
     }
 }
 
@@ -188,11 +202,11 @@ impl<T> Len for BitPackedSearch<'_, T> {
 
 #[cfg(test)]
 mod test {
-    use vortex::array::PrimitiveArray;
-    use vortex::compute::{
-        search_sorted, search_sorted_many, slice, SearchResult, SearchSortedFn, SearchSortedSide,
+    use vortex_array::array::PrimitiveArray;
+    use vortex_array::compute::{
+        search_sorted, search_sorted_many, slice, SearchResult, SearchSortedSide,
     };
-    use vortex::IntoArray;
+    use vortex_array::IntoArrayData;
     use vortex_dtype::Nullability;
     use vortex_scalar::Scalar;
 
@@ -251,12 +265,12 @@ mod test {
         )
         .unwrap();
 
-        let found = bitpacked
-            .search_sorted(
-                &Scalar::primitive(1u64, Nullability::Nullable),
-                SearchSortedSide::Left,
-            )
-            .unwrap();
+        let found = search_sorted(
+            bitpacked.as_ref(),
+            Scalar::primitive(1u64, Nullability::Nullable),
+            SearchSortedSide::Left,
+        )
+        .unwrap();
         assert_eq!(found, SearchResult::Found(0));
     }
 
@@ -281,11 +295,7 @@ mod test {
         let results = search_sorted_many(
             bitpacked.as_ref(),
             &[3u64, 2u64, 1u64],
-            &[
-                SearchSortedSide::Left,
-                SearchSortedSide::Left,
-                SearchSortedSide::Left,
-            ],
+            SearchSortedSide::Left,
         )
         .unwrap();
 
