@@ -1,18 +1,21 @@
+use std::fmt::{Debug, Display};
+use std::sync::Arc;
+
 use ::serde::{Deserialize, Serialize};
 pub use compress::*;
 use fastlanes::BitPacking;
-use vortex::array::{PrimitiveArray, SparseArray};
-use vortex::encoding::ids;
-use vortex::stats::{ArrayStatisticsCompute, StatsSet};
-use vortex::validity::{ArrayValidity, LogicalValidity, Validity, ValidityMetadata};
-use vortex::variants::{ArrayVariants, PrimitiveArrayTrait};
-use vortex::visitor::{AcceptArrayVisitor, ArrayVisitor};
-use vortex::{impl_encoding, Array, ArrayDType, ArrayTrait, Canonical, IntoCanonical, TypedArray};
+use vortex_array::array::{PrimitiveArray, SparseArray};
+use vortex_array::encoding::ids;
+use vortex_array::stats::{StatisticsVTable, StatsSet};
+use vortex_array::validity::{LogicalValidity, Validity, ValidityMetadata, ValidityVTable};
+use vortex_array::variants::{ArrayVariants, PrimitiveArrayTrait};
+use vortex_array::visitor::{ArrayVisitor, VisitorVTable};
+use vortex_array::{
+    impl_encoding, ArrayDType, ArrayData, ArrayLen, ArrayTrait, Canonical, IntoCanonical,
+};
 use vortex_buffer::Buffer;
 use vortex_dtype::{DType, NativePType, Nullability, PType};
-use vortex_error::{
-    vortex_bail, vortex_err, vortex_panic, VortexError, VortexExpect as _, VortexResult,
-};
+use vortex_error::{vortex_bail, vortex_err, VortexExpect as _, VortexResult};
 
 mod compress;
 mod compute;
@@ -21,14 +24,18 @@ impl_encoding!("fastlanes.bitpacked", ids::FL_BITPACKED, BitPacked);
 
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct BitPackedMetadata {
-    // TODO(ngates): serialize into compact form
     validity: ValidityMetadata,
-    bit_width: usize,
-    offset: usize, // Know to be <1024
-    length: usize, // Store end padding instead <1024
+    bit_width: u8,
+    offset: u16, // must be <1024
     has_patches: bool,
 }
 
+impl Display for BitPackedMetadata {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        Debug::fmt(self, f)
+    }
+}
+
 /// NB: All non-null values in the patches array are considered patches
 impl BitPackedArray {
     /// Create a new bitpacked array using a buffer of packed data.
@@ -38,8 +45,8 @@ impl BitPackedArray {
         packed: Buffer,
         ptype: PType,
         validity: Validity,
-        patches: Option<Array>,
-        bit_width: usize,
+        patches: Option<ArrayData>,
+        bit_width: u8,
         len: usize,
     ) -> VortexResult<Self> {
         Self::try_new_from_offset(packed, ptype, validity, patches, bit_width, len, 0)
@@ -49,10 +56,10 @@ impl BitPackedArray {
         packed: Buffer,
         ptype: PType,
         validity: Validity,
-        patches: Option<Array>,
-        bit_width: usize,
+        patches: Option<ArrayData>,
+        bit_width: u8,
         length: usize,
-        offset: usize,
+        offset: u16,
     ) -> VortexResult<Self> {
         let dtype = DType::Primitive(ptype, validity.nullability());
 
@@ -60,7 +67,7 @@ impl BitPackedArray {
             vortex_bail!(MismatchedTypes: "uint", &dtype);
         }
 
-        if bit_width > u64::BITS as usize {
+        if bit_width > u64::BITS as u8 {
             vortex_bail!("Unsupported bit width {}", bit_width);
         }
         if offset > 1023 {
@@ -71,7 +78,8 @@ impl BitPackedArray {
         }
 
         // expected packed size is in bytes
-        let expected_packed_size = ((length + offset + 1023) / 1024) * (128 * bit_width);
+        let expected_packed_size =
+            ((length + offset as usize + 1023) / 1024) * (128 * bit_width as usize);
         if packed.len() != expected_packed_size {
             return Err(vortex_err!(
                 "Expected {} packed bytes, got {}",
@@ -90,7 +98,7 @@ impl BitPackedArray {
                 )
             }
 
-            if SparseArray::try_from(parray)?.indices().is_empty() {
+            if SparseArray::try_from(parray.clone())?.indices().is_empty() {
                 vortex_bail!("cannot construct BitPackedArray using patches without indices");
             }
         }
@@ -98,7 +106,6 @@ impl BitPackedArray {
         let metadata = BitPackedMetadata {
             validity: validity.to_metadata(length)?,
             offset,
-            length,
             bit_width,
             has_patches: patches.is_some(),
         };
@@ -111,16 +118,16 @@ impl BitPackedArray {
             children.push(a)
         }
 
-        Ok(Self {
-            typed: TypedArray::try_from_parts(
-                dtype,
-                length,
-                metadata,
-                Some(packed),
-                children.into(),
-                StatsSet::new(),
-            )?,
-        })
+        ArrayData::try_new_owned(
+            &BitPackedEncoding,
+            dtype,
+            length,
+            Arc::new(metadata),
+            Some(packed),
+            children.into(),
+            StatsSet::default(),
+        )?
+        .try_into()
     }
 
     #[inline]
@@ -145,7 +152,7 @@ impl BitPackedArray {
     }
 
     #[inline]
-    pub fn bit_width(&self) -> usize {
+    pub fn bit_width(&self) -> u8 {
         self.metadata().bit_width
     }
 
@@ -154,7 +161,7 @@ impl BitPackedArray {
     /// If present, patches MUST be a `SparseArray` with equal-length to this array, and whose
     /// indices indicate the locations of patches. The indices must have non-zero length.
     #[inline]
-    pub fn patches(&self) -> Option<Array> {
+    pub fn patches(&self) -> Option<ArrayData> {
         self.metadata().has_patches.then(|| {
             self.as_ref()
                 .child(
@@ -167,7 +174,7 @@ impl BitPackedArray {
     }
 
     #[inline]
-    pub fn offset(&self) -> usize {
+    pub fn offset(&self) -> u16 {
         self.metadata().offset
     }
 
@@ -181,25 +188,14 @@ impl BitPackedArray {
         })
     }
 
-    pub fn encode(array: &Array, bit_width: usize) -> VortexResult<Self> {
-        if let Ok(parray) = PrimitiveArray::try_from(array) {
+    pub fn encode(array: &ArrayData, bit_width: u8) -> VortexResult<Self> {
+        if let Ok(parray) = PrimitiveArray::try_from(array.clone()) {
             bitpack_encode(parray, bit_width)
         } else {
             vortex_bail!("Bitpacking can only encode primitive arrays");
         }
     }
 
-    #[inline]
-    pub fn ptype(&self) -> PType {
-        self.dtype().try_into().unwrap_or_else(|err: VortexError| {
-            vortex_panic!(
-                err,
-                "Failed to convert BitpackedArray DType {} to PType",
-                self.dtype()
-            )
-        })
-    }
-
     #[inline]
     pub fn max_packed_value(&self) -> usize {
         (1 << self.bit_width()) - 1
@@ -212,35 +208,29 @@ impl IntoCanonical for BitPackedArray {
     }
 }
 
-impl ArrayValidity for BitPackedArray {
-    fn is_valid(&self, index: usize) -> bool {
-        self.validity().is_valid(index)
+impl ValidityVTable<BitPackedArray> for BitPackedEncoding {
+    fn is_valid(&self, array: &BitPackedArray, index: usize) -> bool {
+        array.validity().is_valid(index)
     }
 
-    fn logical_validity(&self) -> LogicalValidity {
-        self.validity().to_logical(self.len())
+    fn logical_validity(&self, array: &BitPackedArray) -> LogicalValidity {
+        array.validity().to_logical(array.len())
     }
 }
 
-impl AcceptArrayVisitor for BitPackedArray {
-    fn accept(&self, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
-        visitor.visit_buffer(self.packed())?;
-        if let Some(patches) = self.patches().as_ref() {
+impl VisitorVTable<BitPackedArray> for BitPackedEncoding {
+    fn accept(&self, array: &BitPackedArray, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
+        visitor.visit_buffer(array.packed())?;
+        if let Some(patches) = array.patches().as_ref() {
             visitor.visit_child("patches", patches)?;
         }
-        visitor.visit_validity(&self.validity())
+        visitor.visit_validity(&array.validity())
     }
 }
 
-impl ArrayStatisticsCompute for BitPackedArray {}
+impl StatisticsVTable<BitPackedArray> for BitPackedEncoding {}
 
-impl ArrayTrait for BitPackedArray {
-    fn nbytes(&self) -> usize {
-        // Ignore any overheads like padding or the bit-width flag.
-        let packed_size = ((self.bit_width() * self.len()) + 7) / 8;
-        packed_size + self.patches().map(|p| p.nbytes()).unwrap_or(0)
-    }
-}
+impl ArrayTrait for BitPackedArray {}
 
 impl ArrayVariants for BitPackedArray {
     fn as_primitive_array(&self) -> Option<&dyn PrimitiveArrayTrait> {
@@ -252,8 +242,8 @@ impl PrimitiveArrayTrait for BitPackedArray {}
 
 #[cfg(test)]
 mod test {
-    use vortex::array::PrimitiveArray;
-    use vortex::{IntoArray, IntoArrayVariant};
+    use vortex_array::array::PrimitiveArray;
+    use vortex_array::{IntoArrayData, IntoArrayVariant};
 
     use crate::BitPackedArray;
 
