@@ -1,11 +1,12 @@
 use arrayref::{array_mut_ref, array_ref};
 use fastlanes::{Delta, Transpose};
 use num_traits::{WrappingAdd, WrappingSub};
-use vortex::array::PrimitiveArray;
-use vortex::compute::unary::fill_forward;
-use vortex::compute::SliceFn;
-use vortex::validity::Validity;
-use vortex::IntoArrayVariant;
+use vortex_array::array::PrimitiveArray;
+use vortex_array::compute::slice;
+use vortex_array::compute::unary::fill_forward;
+use vortex_array::validity::Validity;
+use vortex_array::variants::PrimitiveArrayTrait;
+use vortex_array::{ArrayLen, IntoArrayVariant};
 use vortex_dtype::{match_each_unsigned_integer_ptype, NativePType, Nullability};
 use vortex_error::VortexResult;
 
@@ -107,9 +108,7 @@ pub fn delta_decompress(array: DeltaArray) -> VortexResult<PrimitiveArray> {
         )
     });
 
-    decoded
-        .slice(array.offset(), array.offset() + array.len())?
-        .into_primitive()
+    slice(decoded, array.offset(), array.offset() + array.len())?.into_primitive()
 }
 
 fn decompress_primitive<T: NativePType + Delta + Transpose + WrappingAdd>(
@@ -167,6 +166,8 @@ where
 
 #[cfg(test)]
 mod test {
+    use vortex_array::ArrayLen;
+
     use super::*;
 
     #[test]
