@@ -1,98 +1,111 @@
-use std::ops::{AddAssign, Shl, Shr};
+use std::ops::AddAssign;
 
-use num_traits::{WrappingAdd, WrappingSub};
-use vortex::compute::unary::{scalar_at_unchecked, ScalarAtFn};
-use vortex::compute::{
-    filter, search_sorted, slice, take, ArrayCompute, FilterFn, SearchResult, SearchSortedFn,
-    SearchSortedSide, SliceFn, TakeFn,
+use num_traits::{CheckedShl, CheckedShr, WrappingAdd, WrappingSub};
+use vortex_array::compute::unary::{scalar_at, ScalarAtFn};
+use vortex_array::compute::{
+    filter, search_sorted, slice, take, ComputeVTable, FilterFn, FilterMask, SearchResult,
+    SearchSortedFn, SearchSortedSide, SliceFn, TakeFn, TakeOptions,
 };
-use vortex::{Array, ArrayDType, IntoArray};
+use vortex_array::variants::PrimitiveArrayTrait;
+use vortex_array::{ArrayDType, ArrayData, IntoArrayData};
 use vortex_dtype::{match_each_integer_ptype, NativePType};
 use vortex_error::{VortexError, VortexExpect as _, VortexResult};
-use vortex_scalar::{PValue, PrimitiveScalar, Scalar};
+use vortex_scalar::{PValue, Scalar};
 
-use crate::FoRArray;
+use crate::{FoRArray, FoREncoding};
 
-impl ArrayCompute for FoRArray {
-    fn scalar_at(&self) -> Option<&dyn ScalarAtFn> {
+impl ComputeVTable for FoREncoding {
+    fn filter_fn(&self) -> Option<&dyn FilterFn<ArrayData>> {
         Some(self)
     }
 
-    fn search_sorted(&self) -> Option<&dyn SearchSortedFn> {
+    fn scalar_at_fn(&self) -> Option<&dyn ScalarAtFn<ArrayData>> {
         Some(self)
     }
 
-    fn slice(&self) -> Option<&dyn SliceFn> {
+    fn search_sorted_fn(&self) -> Option<&dyn SearchSortedFn<ArrayData>> {
         Some(self)
     }
 
-    fn take(&self) -> Option<&dyn TakeFn> {
+    fn slice_fn(&self) -> Option<&dyn SliceFn<ArrayData>> {
         Some(self)
     }
 
-    fn filter(&self) -> Option<&dyn FilterFn> {
+    fn take_fn(&self) -> Option<&dyn TakeFn<ArrayData>> {
         Some(self)
     }
 }
 
-impl TakeFn for FoRArray {
-    fn take(&self, indices: &Array) -> VortexResult<Array> {
-        Self::try_new(
-            take(self.encoded(), indices)?,
-            self.reference().clone(),
-            self.shift(),
+impl TakeFn<FoRArray> for FoREncoding {
+    fn take(
+        &self,
+        array: &FoRArray,
+        indices: &ArrayData,
+        options: TakeOptions,
+    ) -> VortexResult<ArrayData> {
+        FoRArray::try_new(
+            take(array.encoded(), indices, options)?,
+            array.reference_scalar(),
+            array.shift(),
         )
         .map(|a| a.into_array())
     }
 }
 
-impl FilterFn for FoRArray {
-    fn filter(&self, predicate: &Array) -> VortexResult<Array> {
-        Self::try_new(
-            filter(self.encoded(), predicate)?,
-            self.reference().clone(),
-            self.shift(),
+impl FilterFn<FoRArray> for FoREncoding {
+    fn filter(&self, array: &FoRArray, mask: FilterMask) -> VortexResult<ArrayData> {
+        FoRArray::try_new(
+            filter(&array.encoded(), mask)?,
+            array.reference_scalar(),
+            array.shift(),
         )
         .map(|a| a.into_array())
     }
 }
 
-impl ScalarAtFn for FoRArray {
-    fn scalar_at(&self, index: usize) -> VortexResult<Scalar> {
-        Ok(self.scalar_at_unchecked(index))
-    }
+impl ScalarAtFn<FoRArray> for FoREncoding {
+    fn scalar_at(&self, array: &FoRArray, index: usize) -> VortexResult<Scalar> {
+        let encoded_pvalue = scalar_at(array.encoded(), index)?.reinterpret_cast(array.ptype());
+        let encoded_pvalue = encoded_pvalue.as_primitive();
+        let reference = array.reference_scalar();
+        let reference = reference.as_primitive();
 
-    fn scalar_at_unchecked(&self, index: usize) -> Scalar {
-        let encoded_scalar =
-            scalar_at_unchecked(self.encoded(), index).reinterpret_cast(self.ptype());
-        let encoded =
-            PrimitiveScalar::try_from(&encoded_scalar).vortex_expect("Invalid encoded scalar");
-        let reference =
-            PrimitiveScalar::try_from(self.reference()).vortex_expect("Invalid reference scalar");
-
-        match_each_integer_ptype!(encoded.ptype(), |$P| {
-            encoded.typed_value::<$P>().map(|v| (v << self.shift()).wrapping_add(reference.typed_value::<$P>().unwrap()))
-                    .map(|v| Scalar::primitive::<$P>(v, encoded.dtype().nullability()))
-                    .unwrap_or_else(|| Scalar::null(encoded.dtype().clone()))
-        })
+        Ok(match_each_integer_ptype!(array.ptype(), |$P| {
+            encoded_pvalue
+                .typed_value::<$P>()
+                .map(|v|
+                     v.checked_shl(array.shift() as u32)
+                     .unwrap_or_default()
+                     .wrapping_add(
+                         reference
+                             .typed_value::<$P>()
+                             .vortex_expect("FoRArray Reference value cannot be null")))
+                .map(|v| Scalar::primitive::<$P>(v, array.dtype().nullability()))
+                .unwrap_or_else(|| Scalar::null(array.dtype().clone()))
+        }))
     }
 }
 
-impl SliceFn for FoRArray {
-    fn slice(&self, start: usize, stop: usize) -> VortexResult<Array> {
-        Self::try_new(
-            slice(self.encoded(), start, stop)?,
-            self.reference().clone(),
-            self.shift(),
+impl SliceFn<FoRArray> for FoREncoding {
+    fn slice(&self, array: &FoRArray, start: usize, stop: usize) -> VortexResult<ArrayData> {
+        FoRArray::try_new(
+            slice(array.encoded(), start, stop)?,
+            array.reference_scalar(),
+            array.shift(),
         )
         .map(|a| a.into_array())
     }
 }
 
-impl SearchSortedFn for FoRArray {
-    fn search_sorted(&self, value: &Scalar, side: SearchSortedSide) -> VortexResult<SearchResult> {
-        match_each_integer_ptype!(self.ptype(), |$P| {
-            search_sorted_typed::<$P>(self, value, side)
+impl SearchSortedFn<FoRArray> for FoREncoding {
+    fn search_sorted(
+        &self,
+        array: &FoRArray,
+        value: &Scalar,
+        side: SearchSortedSide,
+    ) -> VortexResult<SearchResult> {
+        match_each_integer_ptype!(array.ptype(), |$P| {
+            search_sorted_typed::<$P>(array, value, side)
         })
     }
 }
@@ -105,14 +118,19 @@ fn search_sorted_typed<T>(
 where
     T: NativePType
         + for<'a> TryFrom<&'a Scalar, Error = VortexError>
-        + Shr<u8, Output = T>
-        + Shl<u8, Output = T>
+        + TryFrom<PValue, Error = VortexError>
+        + CheckedShr
+        + CheckedShl
         + WrappingSub
         + WrappingAdd
         + AddAssign
         + Into<PValue>,
 {
-    let min: T = array.reference().try_into()?;
+    let min: T = array
+        .reference_scalar()
+        .as_primitive()
+        .typed_value::<T>()
+        .vortex_expect("Reference value cannot be null");
     let primitive_value: T = value.cast(array.dtype())?.as_ref().try_into()?;
     // Make sure that smaller values are still smaller and not larger than (which they would be after wrapping_sub)
     if primitive_value < min {
@@ -121,8 +139,14 @@ where
 
     // When the values in the array are shifted, not all values in the domain are representable in the compressed
     // space. Multiple different search values can translate to same value in the compressed space.
-    let encoded_value = primitive_value.wrapping_sub(&min) >> array.shift();
-    let decoded_value = (encoded_value << array.shift()).wrapping_add(&min);
+    let encoded_value = primitive_value
+        .wrapping_sub(&min)
+        .checked_shr(array.shift() as u32)
+        .unwrap_or_default();
+    let decoded_value = encoded_value
+        .checked_shl(array.shift() as u32)
+        .unwrap_or_default()
+        .wrapping_add(&min);
 
     // We first determine whether the value can be represented in the compressed array. For any value that is not
     // representable, it is by definition NotFound. For NotFound values, the correct insertion index is by definition
@@ -153,23 +177,27 @@ where
 
 #[cfg(test)]
 mod test {
-    use vortex::array::PrimitiveArray;
-    use vortex::compute::unary::scalar_at;
-    use vortex::compute::{search_sorted, SearchResult, SearchSortedSide};
+    use vortex_array::array::PrimitiveArray;
+    use vortex_array::compute::unary::scalar_at;
+    use vortex_array::compute::{search_sorted, SearchResult, SearchSortedSide};
+    use vortex_array::IntoArrayData;
 
     use crate::{for_compress, FoRArray};
 
     #[test]
     fn for_scalar_at() {
-        let for_arr = for_compress(&PrimitiveArray::from(vec![1100, 1500, 1900])).unwrap();
-        assert_eq!(scalar_at(&for_arr, 0).unwrap(), 1100.into());
-        assert_eq!(scalar_at(&for_arr, 1).unwrap(), 1500.into());
-        assert_eq!(scalar_at(&for_arr, 2).unwrap(), 1900.into());
+        let for_arr = for_compress(&PrimitiveArray::from(vec![-100, 1100, 1500, 1900])).unwrap();
+        assert_eq!(scalar_at(&for_arr, 0).unwrap(), (-100).into());
+        assert_eq!(scalar_at(&for_arr, 1).unwrap(), 1100.into());
+        assert_eq!(scalar_at(&for_arr, 2).unwrap(), 1500.into());
+        assert_eq!(scalar_at(&for_arr, 3).unwrap(), 1900.into());
     }
 
     #[test]
     fn for_search() {
-        let for_arr = for_compress(&PrimitiveArray::from(vec![1100, 1500, 1900])).unwrap();
+        let for_arr = for_compress(&PrimitiveArray::from(vec![1100, 1500, 1900]))
+            .unwrap()
+            .into_array();
         assert_eq!(
             search_sorted(&for_arr, 1500, SearchSortedSide::Left).unwrap(),
             SearchResult::Found(1)
@@ -186,17 +214,17 @@ mod test {
 
     #[test]
     fn search_with_shift_notfound() {
-        let for_arr = for_compress(&PrimitiveArray::from(vec![62, 114])).unwrap();
+        let for_arr = for_compress(&PrimitiveArray::from(vec![62, 114]))
+            .unwrap()
+            .into_array();
         assert_eq!(
             search_sorted(&for_arr, 63, SearchSortedSide::Left).unwrap(),
             SearchResult::NotFound(1)
         );
-        let for_arr = for_compress(&PrimitiveArray::from(vec![62, 114])).unwrap();
         assert_eq!(
             search_sorted(&for_arr, 61, SearchSortedSide::Left).unwrap(),
             SearchResult::NotFound(0)
         );
-        let for_arr = for_compress(&PrimitiveArray::from(vec![62, 114])).unwrap();
         assert_eq!(
             search_sorted(&for_arr, 113, SearchSortedSide::Left).unwrap(),
             SearchResult::NotFound(1)
@@ -209,10 +237,16 @@ mod test {
 
     #[test]
     fn search_with_shift_repeated() {
-        let arr = for_compress(&PrimitiveArray::from(vec![62, 62, 114, 114])).unwrap();
+        let arr = for_compress(&PrimitiveArray::from(vec![62, 62, 114, 114]))
+            .unwrap()
+            .into_array();
         let for_array = FoRArray::try_from(arr.clone()).unwrap();
 
-        let min: i32 = for_array.reference().try_into().unwrap();
+        let min: i32 = for_array
+            .reference_scalar()
+            .as_primitive()
+            .typed_value::<i32>()
+            .unwrap();
         assert_eq!(min, 62);
         assert_eq!(for_array.shift(), 1);
 
