@@ -1,14 +1,17 @@
+use std::fmt::{Debug, Display};
 use std::sync::Arc;
 
 use fsst::{Decompressor, Symbol};
 use serde::{Deserialize, Serialize};
-use vortex::array::VarBinArray;
-use vortex::encoding::ids;
-use vortex::stats::{ArrayStatisticsCompute, StatsSet};
-use vortex::validity::{ArrayValidity, LogicalValidity, Validity};
-use vortex::variants::{ArrayVariants, BinaryArrayTrait, Utf8ArrayTrait};
-use vortex::visitor::AcceptArrayVisitor;
-use vortex::{impl_encoding, Array, ArrayDType, ArrayTrait, IntoCanonical};
+use vortex_array::array::{VarBin, VarBinArray};
+use vortex_array::encoding::ids;
+use vortex_array::stats::{StatisticsVTable, StatsSet};
+use vortex_array::validity::{ArrayValidity, LogicalValidity, Validity, ValidityVTable};
+use vortex_array::variants::{ArrayVariants, BinaryArrayTrait, Utf8ArrayTrait};
+use vortex_array::visitor::{ArrayVisitor, VisitorVTable};
+use vortex_array::{
+    impl_encoding, ArrayDType, ArrayData, ArrayDef, ArrayLen, ArrayTrait, IntoCanonical,
+};
 use vortex_dtype::{DType, Nullability, PType};
 use vortex_error::{vortex_bail, VortexExpect, VortexResult};
 
@@ -20,8 +23,14 @@ static SYMBOL_LENS_DTYPE: DType = DType::Primitive(PType::U8, Nullability::NonNu
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct FSSTMetadata {
     symbols_len: usize,
-    codes_dtype: DType,
-    uncompressed_lengths_dtype: DType,
+    codes_nullability: Nullability,
+    uncompressed_lengths_ptype: PType,
+}
+
+impl Display for FSSTMetadata {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        Debug::fmt(self, f)
+    }
 }
 
 impl FSSTArray {
@@ -35,10 +44,10 @@ impl FSSTArray {
     /// which tells the decoder to emit the following byte without doing a table lookup.
     pub fn try_new(
         dtype: DType,
-        symbols: Array,
-        symbol_lengths: Array,
-        codes: Array,
-        uncompressed_lengths: Array,
+        symbols: ArrayData,
+        symbol_lengths: ArrayData,
+        codes: ArrayData,
+        uncompressed_lengths: ArrayData,
     ) -> VortexResult<Self> {
         // Check: symbols must be a u64 array
         if symbols.dtype() != &SYMBOLS_DTYPE {
@@ -62,8 +71,15 @@ impl FSSTArray {
             vortex_bail!(InvalidArgument: "uncompressed_lengths must be same len as codes");
         }
 
-        if !uncompressed_lengths.dtype().is_int() {
-            vortex_bail!(InvalidArgument: "uncompressed_lengths must have integer type");
+        if !uncompressed_lengths.dtype().is_int() || uncompressed_lengths.dtype().is_nullable() {
+            vortex_bail!(InvalidArgument: "uncompressed_lengths must have integer type and cannot be nullable");
+        }
+
+        if codes.encoding().id() != VarBin::ID {
+            vortex_bail!(
+                InvalidArgument: "codes must have varbin encoding, was {}",
+                codes.encoding().id()
+            );
         }
 
         // Check: strings must be a Binary array.
@@ -73,8 +89,8 @@ impl FSSTArray {
 
         let symbols_len = symbols.len();
         let len = codes.len();
-        let strings_dtype = codes.dtype().clone();
-        let uncompressed_lengths_dtype = uncompressed_lengths.dtype().clone();
+        let uncompressed_lengths_ptype = PType::try_from(uncompressed_lengths.dtype())?;
+        let codes_nullability = codes.dtype().nullability();
         let children = Arc::new([symbols, symbol_lengths, codes, uncompressed_lengths]);
 
         Self::try_from_parts(
@@ -82,42 +98,57 @@ impl FSSTArray {
             len,
             FSSTMetadata {
                 symbols_len,
-                codes_dtype: strings_dtype,
-                uncompressed_lengths_dtype,
+                codes_nullability,
+                uncompressed_lengths_ptype,
             },
             children,
-            StatsSet::new(),
+            StatsSet::default(),
         )
     }
 
     /// Access the symbol table array
-    pub fn symbols(&self) -> Array {
+    pub fn symbols(&self) -> ArrayData {
         self.as_ref()
             .child(0, &SYMBOLS_DTYPE, self.metadata().symbols_len)
             .vortex_expect("FSSTArray symbols child")
     }
 
     /// Access the symbol table array
-    pub fn symbol_lengths(&self) -> Array {
+    pub fn symbol_lengths(&self) -> ArrayData {
         self.as_ref()
             .child(1, &SYMBOL_LENS_DTYPE, self.metadata().symbols_len)
             .vortex_expect("FSSTArray symbol_lengths child")
     }
 
     /// Access the codes array
-    pub fn codes(&self) -> Array {
+    pub fn codes(&self) -> ArrayData {
         self.as_ref()
-            .child(2, &self.metadata().codes_dtype, self.len())
+            .child(2, &self.codes_dtype(), self.len())
             .vortex_expect("FSSTArray codes child")
     }
 
+    /// Get the DType of the codes array
+    #[inline]
+    pub fn codes_dtype(&self) -> DType {
+        DType::Binary(self.metadata().codes_nullability)
+    }
+
     /// Get the uncompressed length for each element in the array.
-    pub fn uncompressed_lengths(&self) -> Array {
+    pub fn uncompressed_lengths(&self) -> ArrayData {
         self.as_ref()
-            .child(3, &self.metadata().uncompressed_lengths_dtype, self.len())
+            .child(3, &self.uncompressed_lengths_dtype(), self.len())
             .vortex_expect("FSST uncompressed_lengths child")
     }
 
+    /// Get the DType of the uncompressed lengths array
+    #[inline]
+    pub fn uncompressed_lengths_dtype(&self) -> DType {
+        DType::Primitive(
+            self.metadata().uncompressed_lengths_ptype,
+            Nullability::NonNullable,
+        )
+    }
+
     /// Get the validity for this array.
     pub fn validity(&self) -> Validity {
         VarBinArray::try_from(self.codes())
@@ -160,24 +191,24 @@ impl FSSTArray {
     }
 }
 
-impl AcceptArrayVisitor for FSSTArray {
-    fn accept(&self, visitor: &mut dyn vortex::visitor::ArrayVisitor) -> VortexResult<()> {
-        visitor.visit_child("symbols", &self.symbols())?;
-        visitor.visit_child("symbol_lengths", &self.symbol_lengths())?;
-        visitor.visit_child("codes", &self.codes())?;
-        visitor.visit_child("uncompressed_lengths", &self.uncompressed_lengths())
+impl VisitorVTable<FSSTArray> for FSSTEncoding {
+    fn accept(&self, array: &FSSTArray, visitor: &mut dyn ArrayVisitor) -> VortexResult<()> {
+        visitor.visit_child("symbols", &array.symbols())?;
+        visitor.visit_child("symbol_lengths", &array.symbol_lengths())?;
+        visitor.visit_child("codes", &array.codes())?;
+        visitor.visit_child("uncompressed_lengths", &array.uncompressed_lengths())
     }
 }
 
-impl ArrayStatisticsCompute for FSSTArray {}
+impl StatisticsVTable<FSSTArray> for FSSTEncoding {}
 
-impl ArrayValidity for FSSTArray {
-    fn is_valid(&self, index: usize) -> bool {
-        self.codes().with_dyn(|a| a.is_valid(index))
+impl ValidityVTable<FSSTArray> for FSSTEncoding {
+    fn is_valid(&self, array: &FSSTArray, index: usize) -> bool {
+        array.codes().is_valid(index)
     }
 
-    fn logical_validity(&self) -> LogicalValidity {
-        self.codes().with_dyn(|a| a.logical_validity())
+    fn logical_validity(&self, array: &FSSTArray) -> LogicalValidity {
+        array.codes().logical_validity()
     }
 }
 
