@@ -1,5 +1,6 @@
 use croaring::Bitmap;
-use vortex::array::BoolArray;
+use vortex_array::array::BoolArray;
+use vortex_array::ArrayLen;
 use vortex_error::VortexResult;
 
 use crate::RoaringBoolArray;
