@@ -4,8 +4,8 @@ use criterion::{black_box, criterion_group, criterion_main, Criterion};
 use itertools::Itertools;
 use rand::distributions::Uniform;
 use rand::{thread_rng, Rng};
-use vortex::array::ChunkedArray;
-use vortex::IntoArray;
+use vortex_array::array::ChunkedArray;
+use vortex_array::IntoArrayData;
 use vortex_error::VortexError;
 
 fn scalar_subtract(c: &mut Criterion) {
@@ -29,7 +29,8 @@ fn scalar_subtract(c: &mut Criterion) {
     group.bench_function("vortex", |b| {
         b.iter(|| {
             let array =
-                vortex::compute::unary::subtract_scalar(&chunked, &to_subtract.into()).unwrap();
+                vortex_array::compute::unary::subtract_scalar(&chunked, &to_subtract.into())
+                    .unwrap();
 
             let chunked = ChunkedArray::try_from(array).unwrap();
             black_box(chunked);
