@@ -7,7 +7,7 @@ use crate::variants::{
     ArrayVariants, BinaryArrayTrait, BoolArrayTrait, ExtensionArrayTrait, ListArrayTrait,
     NullArrayTrait, PrimitiveArrayTrait, StructArrayTrait, Utf8ArrayTrait,
 };
-use crate::{Array, ArrayDType, IntoArray};
+use crate::{ArrayDType, ArrayData, IntoArrayData};
 
 /// Chunked arrays support all DTypes
 impl ArrayVariants for ChunkedArray {
@@ -47,12 +47,18 @@ impl ArrayVariants for ChunkedArray {
 impl NullArrayTrait for ChunkedArray {}
 
 impl BoolArrayTrait for ChunkedArray {
-    fn maybe_null_indices_iter(&self) -> Box<dyn Iterator<Item = usize>> {
-        todo!()
-    }
-
-    fn maybe_null_slices_iter(&self) -> Box<dyn Iterator<Item = (usize, usize)>> {
-        todo!()
+    fn invert(&self) -> VortexResult<ArrayData> {
+        let chunks = self
+            .chunks()
+            .map(|c| {
+                c.with_dyn(|a| {
+                    a.as_bool_array()
+                        .ok_or_else(|| vortex_err!("Child was not a bool array"))
+                        .and_then(|b| b.invert())
+                })
+            })
+            .collect::<VortexResult<Vec<_>>>()?;
+        ChunkedArray::try_new(chunks, self.dtype().clone()).map(|a| a.into_array())
     }
 }
 
@@ -63,7 +69,7 @@ impl Utf8ArrayTrait for ChunkedArray {}
 impl BinaryArrayTrait for ChunkedArray {}
 
 impl StructArrayTrait for ChunkedArray {
-    fn field(&self, idx: usize) -> Option<Array> {
+    fn field(&self, idx: usize) -> Option<ArrayData> {
         let mut chunks = Vec::with_capacity(self.nchunks());
         for chunk in self.chunks() {
             chunks.push(chunk.with_dyn(|a| a.as_struct_array().and_then(|s| s.field(idx)))?);
@@ -82,7 +88,7 @@ impl StructArrayTrait for ChunkedArray {
         Some(chunked)
     }
 
-    fn project(&self, projection: &[Field]) -> VortexResult<Array> {
+    fn project(&self, projection: &[Field]) -> VortexResult<ArrayData> {
         let mut chunks = Vec::with_capacity(self.nchunks());
         for chunk in self.chunks() {
             chunks.push(chunk.with_dyn(|a| {
@@ -108,10 +114,10 @@ impl StructArrayTrait for ChunkedArray {
 impl ListArrayTrait for ChunkedArray {}
 
 impl ExtensionArrayTrait for ChunkedArray {
-    fn storage_array(&self) -> Array {
+    fn storage_data(&self) -> ArrayData {
         ChunkedArray::from_iter(
             self.chunks()
-                .map(|chunk| chunk.with_dyn(|a| a.as_extension_array_unchecked().storage_array())),
+                .map(|chunk| chunk.with_dyn(|a| a.as_extension_array_unchecked().storage_data())),
         )
         .into_array()
     }
