@@ -1,13 +1,15 @@
 #[cfg(test)]
 mod test;
 
+use std::sync::Arc;
+
 use vortex_datetime_dtype::{TemporalMetadata, TimeUnit, DATE_ID, TIMESTAMP_ID, TIME_ID};
 use vortex_dtype::{DType, ExtDType};
 use vortex_error::{vortex_panic, VortexError};
 
 use crate::array::ExtensionArray;
 use crate::variants::ExtensionArrayTrait;
-use crate::{Array, ArrayDType, IntoArray};
+use crate::{ArrayDType, ArrayData, IntoArrayData};
 
 /// An array wrapper for primitive values that have an associated temporal meaning.
 ///
@@ -67,29 +69,25 @@ impl TemporalArray {
     /// If the time unit is days, and the array is not of primitive I32 type, it panics.
     ///
     /// If any other time unit is provided, it panics.
-    pub fn new_date(array: Array, time_unit: TimeUnit) -> Self {
-        let ext_dtype = match time_unit {
+    pub fn new_date(array: ArrayData, time_unit: TimeUnit) -> Self {
+        match time_unit {
             TimeUnit::D => {
                 assert_width!(i32, array);
-
-                ExtDType::new(
-                    DATE_ID.clone(),
-                    Some(TemporalMetadata::Date(time_unit).into()),
-                )
             }
             TimeUnit::Ms => {
                 assert_width!(i64, array);
-
-                ExtDType::new(
-                    DATE_ID.clone(),
-                    Some(TemporalMetadata::Date(time_unit).into()),
-                )
             }
             _ => vortex_panic!("invalid TimeUnit {time_unit} for vortex.date"),
         };
 
+        let ext_dtype = ExtDType::new(
+            DATE_ID.clone(),
+            Arc::new(array.dtype().clone()),
+            Some(TemporalMetadata::Date(time_unit).into()),
+        );
+
         Self {
-            ext: ExtensionArray::new(ext_dtype, array),
+            ext: ExtensionArray::new(Arc::new(ext_dtype), array),
             temporal_metadata: TemporalMetadata::Date(time_unit),
         }
     }
@@ -113,7 +111,7 @@ impl TemporalArray {
     /// If the time unit is microseconds, and the array is not of primitive I64 type, it panics.
     ///
     /// If the time unit is nanoseconds, and the array is not of primitive I64 type, it panics.
-    pub fn new_time(array: Array, time_unit: TimeUnit) -> Self {
+    pub fn new_time(array: ArrayData, time_unit: TimeUnit) -> Self {
         match time_unit {
             TimeUnit::S | TimeUnit::Ms => assert_width!(i32, array),
             TimeUnit::Us | TimeUnit::Ns => assert_width!(i64, array),
@@ -123,7 +121,11 @@ impl TemporalArray {
         let temporal_metadata = TemporalMetadata::Time(time_unit);
         Self {
             ext: ExtensionArray::new(
-                ExtDType::new(TIME_ID.clone(), Some(temporal_metadata.clone().into())),
+                Arc::new(ExtDType::new(
+                    TIME_ID.clone(),
+                    Arc::new(array.dtype().clone()),
+                    Some(temporal_metadata.clone().into()),
+                )),
                 array,
             ),
             temporal_metadata,
@@ -138,14 +140,18 @@ impl TemporalArray {
     /// If `array` does not hold Primitive i64 data, the function will panic.
     ///
     /// If the time_unit is days, the function will panic.
-    pub fn new_timestamp(array: Array, time_unit: TimeUnit, time_zone: Option<String>) -> Self {
+    pub fn new_timestamp(array: ArrayData, time_unit: TimeUnit, time_zone: Option<String>) -> Self {
         assert_width!(i64, array);
 
         let temporal_metadata = TemporalMetadata::Timestamp(time_unit, time_zone);
 
         Self {
             ext: ExtensionArray::new(
-                ExtDType::new(TIMESTAMP_ID.clone(), Some(temporal_metadata.clone().into())),
+                Arc::new(ExtDType::new(
+                    TIMESTAMP_ID.clone(),
+                    Arc::new(array.dtype().clone()),
+                    Some(temporal_metadata.clone().into()),
+                )),
                 array,
             ),
             temporal_metadata,
@@ -158,7 +164,7 @@ impl TemporalArray {
     ///
     /// These values are to be interpreted based on the time unit and optional time-zone stored
     /// in the TemporalMetadata.
-    pub fn temporal_values(&self) -> Array {
+    pub fn temporal_values(&self) -> ArrayData {
         self.ext.storage()
     }
 
@@ -171,18 +177,18 @@ impl TemporalArray {
     }
 
     /// Retrieve the extension DType associated with the underlying array.
-    pub fn ext_dtype(&self) -> &ExtDType {
-        self.ext.ext_dtype()
+    pub fn ext_dtype(&self) -> Arc<ExtDType> {
+        self.ext.ext_dtype().clone()
     }
 }
 
-impl From<TemporalArray> for Array {
+impl From<TemporalArray> for ArrayData {
     fn from(value: TemporalArray) -> Self {
         value.ext.into_array()
     }
 }
 
-impl TryFrom<&Array> for TemporalArray {
+impl TryFrom<ArrayData> for TemporalArray {
     type Error = VortexError;
 
     /// Try to specialize a generic Vortex array as a TemporalArray.
@@ -193,9 +199,9 @@ impl TryFrom<&Array> for TemporalArray {
     ///
     /// If the provided Array does not have recognized ExtMetadata corresponding to one of the known
     /// `TemporalMetadata` variants, an error is returned.
-    fn try_from(value: &Array) -> Result<Self, Self::Error> {
+    fn try_from(value: ArrayData) -> Result<Self, Self::Error> {
         let ext = ExtensionArray::try_from(value)?;
-        let temporal_metadata = TemporalMetadata::try_from(ext.ext_dtype())?;
+        let temporal_metadata = TemporalMetadata::try_from(ext.ext_dtype().as_ref())?;
 
         Ok(Self {
             ext,
@@ -204,17 +210,6 @@ impl TryFrom<&Array> for TemporalArray {
     }
 }
 
-impl TryFrom<Array> for TemporalArray {
-    type Error = VortexError;
-
-    /// Try to specialize a generic Vortex array as a TemporalArray.
-    ///
-    /// Delegates to `TryFrom<&Array>`.
-    fn try_from(value: Array) -> Result<Self, Self::Error> {
-        TemporalArray::try_from(&value)
-    }
-}
-
 // Conversions to/from ExtensionArray
 impl From<&TemporalArray> for ExtensionArray {
     fn from(value: &TemporalArray) -> Self {
@@ -232,7 +227,7 @@ impl TryFrom<ExtensionArray> for TemporalArray {
     type Error = VortexError;
 
     fn try_from(ext: ExtensionArray) -> Result<Self, Self::Error> {
-        let temporal_metadata = TemporalMetadata::try_from(ext.ext_dtype())?;
+        let temporal_metadata = TemporalMetadata::try_from(ext.ext_dtype().as_ref())?;
         Ok(Self {
             ext,
             temporal_metadata,
