@@ -1,4 +1,8 @@
+//! All the built-in encoding schemes and arrays.
+
+#[cfg(test)]
 mod assertions;
+
 mod bool;
 mod chunked;
 mod constant;
@@ -11,6 +15,8 @@ mod struct_;
 mod varbin;
 mod varbinview;
 
+pub mod from;
+
 #[cfg(feature = "arbitrary")]
 pub mod arbitrary;
 
