@@ -2,24 +2,27 @@ use vortex_dtype::{match_each_native_ptype, DType, NativePType, Nullability};
 use vortex_error::{vortex_bail, vortex_err, VortexResult};
 
 use crate::array::primitive::PrimitiveArray;
+use crate::array::PrimitiveEncoding;
 use crate::compute::unary::CastFn;
 use crate::validity::Validity;
-use crate::{Array, ArrayDType, IntoArray};
+use crate::variants::PrimitiveArrayTrait;
+use crate::{ArrayDType, ArrayData, ArrayLen, IntoArrayData};
 
-impl CastFn for PrimitiveArray {
-    fn cast(&self, dtype: &DType) -> VortexResult<Array> {
-        let DType::Primitive(new_ptype, new_nullability) = dtype.clone() else {
+impl CastFn<PrimitiveArray> for PrimitiveEncoding {
+    fn cast(&self, array: &PrimitiveArray, dtype: &DType) -> VortexResult<ArrayData> {
+        let DType::Primitive(new_ptype, new_nullability) = dtype else {
             vortex_bail!(MismatchedTypes: "primitive type", dtype);
         };
+        let (new_ptype, new_nullability) = (*new_ptype, *new_nullability);
 
         // First, check that the cast is compatible with the source array's validity
-        let new_validity = if self.dtype().nullability() == new_nullability {
-            self.validity().clone()
+        let new_validity = if array.dtype().nullability() == new_nullability {
+            array.validity()
         } else if new_nullability == Nullability::Nullable {
             // from non-nullable to nullable
-            self.validity().into_nullable()
+            array.validity().into_nullable()
         } else if new_nullability == Nullability::NonNullable
-            && self.validity().to_logical(self.len()).all_valid()
+            && array.validity().to_logical(array.len()).all_valid()
         {
             // from nullable but all valid, to non-nullable
             Validity::NonNullable
@@ -28,16 +31,17 @@ impl CastFn for PrimitiveArray {
         };
 
         // If the bit width is the same, we can short-circuit and simply update the validity
-        if self.ptype() == new_ptype {
+        if array.ptype() == new_ptype {
             return Ok(
-                PrimitiveArray::new(self.buffer().clone(), self.ptype(), new_validity).into_array(),
+                PrimitiveArray::new(array.buffer().clone(), array.ptype(), new_validity)
+                    .into_array(),
             );
         }
 
         // Otherwise, we need to cast the values one-by-one
         match_each_native_ptype!(new_ptype, |$T| {
             Ok(PrimitiveArray::from_vec(
-                cast::<$T>(self)?,
+                cast::<$T>(array)?,
                 new_validity,
             ).into_array())
         })
@@ -67,42 +71,49 @@ mod test {
     use crate::array::PrimitiveArray;
     use crate::compute::unary::try_cast;
     use crate::validity::Validity;
-    use crate::IntoArray;
+    use crate::{IntoArrayData, IntoArrayVariant};
 
     #[test]
     fn cast_u32_u8() {
         let arr = vec![0u32, 10, 200].into_array();
 
         // cast from u32 to u8
-        let p = try_cast(&arr, PType::U8.into()).unwrap().as_primitive();
+        let p = try_cast(&arr, PType::U8.into())
+            .unwrap()
+            .into_primitive()
+            .unwrap();
         assert_eq!(p.maybe_null_slice::<u8>(), vec![0u8, 10, 200]);
         assert_eq!(p.validity(), Validity::NonNullable);
 
         // to nullable
         let p = try_cast(&p, &DType::Primitive(PType::U8, Nullability::Nullable))
             .unwrap()
-            .as_primitive();
+            .into_primitive()
+            .unwrap();
         assert_eq!(p.maybe_null_slice::<u8>(), vec![0u8, 10, 200]);
         assert_eq!(p.validity(), Validity::AllValid);
 
         // back to non-nullable
         let p = try_cast(&p, &DType::Primitive(PType::U8, Nullability::NonNullable))
             .unwrap()
-            .as_primitive();
+            .into_primitive()
+            .unwrap();
         assert_eq!(p.maybe_null_slice::<u8>(), vec![0u8, 10, 200]);
         assert_eq!(p.validity(), Validity::NonNullable);
 
         // to nullable u32
         let p = try_cast(&p, &DType::Primitive(PType::U32, Nullability::Nullable))
             .unwrap()
-            .as_primitive();
+            .into_primitive()
+            .unwrap();
         assert_eq!(p.maybe_null_slice::<u32>(), vec![0u32, 10, 200]);
         assert_eq!(p.validity(), Validity::AllValid);
 
         // to non-nullable u8
         let p = try_cast(&p, &DType::Primitive(PType::U8, Nullability::NonNullable))
             .unwrap()
-            .as_primitive();
+            .into_primitive()
+            .unwrap();
         assert_eq!(p.maybe_null_slice::<u8>(), vec![0u8, 10, 200]);
         assert_eq!(p.validity(), Validity::NonNullable);
     }
@@ -110,7 +121,10 @@ mod test {
     #[test]
     fn cast_u32_f32() {
         let arr = vec![0u32, 10, 200].into_array();
-        let u8arr = try_cast(&arr, PType::F32.into()).unwrap().as_primitive();
+        let u8arr = try_cast(&arr, PType::F32.into())
+            .unwrap()
+            .into_primitive()
+            .unwrap();
         assert_eq!(u8arr.maybe_null_slice::<f32>(), vec![0.0f32, 10., 200.]);
     }
 
