@@ -1,3 +1,5 @@
+//! Encodings that enable zero-copy sharing of data with Arrow.
+
 use std::sync::Arc;
 
 use arrow_array::types::{
@@ -5,28 +7,29 @@ use arrow_array::types::{
     UInt32Type, UInt64Type, UInt8Type,
 };
 use arrow_array::{
-    ArrayRef, ArrowPrimitiveType, BinaryArray, BooleanArray as ArrowBoolArray, Date32Array,
-    Date64Array, LargeBinaryArray, LargeStringArray, NullArray as ArrowNullArray,
-    PrimitiveArray as ArrowPrimitiveArray, StringArray, StructArray as ArrowStructArray,
-    Time32MillisecondArray, Time32SecondArray, Time64MicrosecondArray, Time64NanosecondArray,
-    TimestampMicrosecondArray, TimestampMillisecondArray, TimestampNanosecondArray,
-    TimestampSecondArray,
+    ArrayRef, ArrowPrimitiveType, BooleanArray as ArrowBoolArray, Date32Array, Date64Array,
+    NullArray as ArrowNullArray, PrimitiveArray as ArrowPrimitiveArray,
+    StructArray as ArrowStructArray, Time32MillisecondArray, Time32SecondArray,
+    Time64MicrosecondArray, Time64NanosecondArray, TimestampMicrosecondArray,
+    TimestampMillisecondArray, TimestampNanosecondArray, TimestampSecondArray,
 };
 use arrow_buffer::ScalarBuffer;
 use arrow_schema::{Field, Fields};
 use vortex_datetime_dtype::{is_temporal_ext_type, TemporalMetadata, TimeUnit};
 use vortex_dtype::{DType, NativePType, PType};
-use vortex_error::{vortex_bail, VortexResult};
+use vortex_error::{vortex_bail, VortexError, VortexResult};
 
 use crate::array::{
-    BoolArray, ExtensionArray, NullArray, PrimitiveArray, StructArray, TemporalArray, VarBinArray,
+    varbinview_as_arrow, BoolArray, ExtensionArray, NullArray, PrimitiveArray, StructArray,
+    TemporalArray, VarBinViewArray,
 };
-use crate::arrow::wrappers::as_offset_buffer;
+use crate::arrow::{infer_data_type, FromArrowArray};
 use crate::compute::unary::try_cast;
-use crate::encoding::ArrayEncoding;
+use crate::encoding::Encoding;
+use crate::stats::ArrayStatistics;
 use crate::validity::ArrayValidity;
-use crate::variants::StructArrayTrait;
-use crate::{Array, ArrayDType, IntoArray, ToArray};
+use crate::variants::{PrimitiveArrayTrait, StructArrayTrait};
+use crate::{ArrayDType, ArrayData, ArrayLen, IntoArrayData};
 
 /// The set of canonical array encodings, also the set of encodings that can be transferred to
 /// Arrow with zero-copy.
@@ -46,21 +49,18 @@ use crate::{Array, ArrayDType, IntoArray, ToArray};
 ///
 /// # Views support
 ///
-/// Binary and String views are a new, better encoding format for nearly all use-cases. For now,
-/// because DataFusion does not include pervasive support for compute over StringView, we opt to use
-/// the [`VarBinArray`] as the canonical encoding (which corresponds to the Arrow `BinaryViewArray`).
+/// Binary and String views, also known as "German strings" are a better encoding format for
+/// nearly all use-cases. Variable-length binary views are part of the Apache Arrow spec, and are
+/// fully supported by the Datafusion query engine. We use them as our canonical string encoding
+/// for all `Utf8` and `Binary` typed arrays in Vortex.
 ///
-/// We expect to change this soon once DataFusion is able to finish up some initial support, which
-/// is tracked in <https://github.com/apache/datafusion/issues/10918>.
 #[derive(Debug, Clone)]
 pub enum Canonical {
     Null(NullArray),
     Bool(BoolArray),
     Primitive(PrimitiveArray),
     Struct(StructArray),
-    VarBin(VarBinArray),
-    // TODO(aduffy): switch to useing VarBinView instead of VarBin
-    // VarBinView(VarBinViewArray),
+    VarBinView(VarBinViewArray),
     Extension(ExtensionArray),
 }
 
@@ -76,17 +76,33 @@ impl Canonical {
             Canonical::Bool(a) => bool_to_arrow(a)?,
             Canonical::Primitive(a) => primitive_to_arrow(a)?,
             Canonical::Struct(a) => struct_to_arrow(a)?,
-            Canonical::VarBin(a) => varbin_to_arrow(a)?,
+            Canonical::VarBinView(a) => varbinview_as_arrow(&a),
             Canonical::Extension(a) => {
-                if !is_temporal_ext_type(a.id()) {
-                    vortex_bail!("unsupported extension dtype with ID {}", a.id().as_ref())
+                if is_temporal_ext_type(a.id()) {
+                    temporal_to_arrow(TemporalArray::try_from(a.into_array())?)?
+                } else {
+                    // Convert storage array directly into arrow, losing type information
+                    // that will let us round-trip.
+                    // TODO(aduffy): https://github.com/spiraldb/vortex/issues/1167
+                    a.storage().into_canonical()?.into_arrow()?
                 }
-                temporal_to_arrow(TemporalArray::try_from(&a.into_array())?)?
             }
         })
     }
 }
 
+impl Canonical {
+    // Create an empty canonical array of the given dtype.
+    pub fn empty(dtype: &DType) -> VortexResult<Canonical> {
+        let arrow_dtype = infer_data_type(dtype)?;
+        ArrayData::from_arrow(
+            arrow_array::new_empty_array(&arrow_dtype),
+            dtype.is_nullable(),
+        )
+        .into_canonical()
+    }
+}
+
 // Unwrap canonical type back down to specialized type.
 impl Canonical {
     pub fn into_null(self) -> VortexResult<NullArray> {
@@ -117,10 +133,10 @@ impl Canonical {
         }
     }
 
-    pub fn into_varbin(self) -> VortexResult<VarBinArray> {
+    pub fn into_varbinview(self) -> VortexResult<VarBinViewArray> {
         match self {
-            Canonical::VarBin(a) => Ok(a),
-            _ => vortex_bail!("Cannot unwrap VarBinArray from {:?}", &self),
+            Canonical::VarBinView(a) => Ok(a),
+            _ => vortex_bail!("Cannot unwrap VarBinViewArray from {:?}", &self),
         }
     }
 
@@ -172,19 +188,11 @@ fn struct_to_arrow(struct_array: StructArray) -> VortexResult<ArrayRef> {
     let field_arrays: Vec<ArrayRef> =
         Iterator::zip(struct_array.names().iter(), struct_array.children())
             .map(|(name, f)| {
-                let canonical = f.into_canonical().map_err(|err| {
-                    err.with_context(format!("Failed to canonicalize field {}", name))
-                })?;
-                match canonical {
-                    // visit nested structs recursively
-                    Canonical::Struct(a) => struct_to_arrow(a),
-                    _ => canonical.into_arrow().map_err(|err| {
-                        err.with_context(format!(
-                            "Failed to convert canonicalized field {} to arrow",
-                            name
-                        ))
-                    }),
-                }
+                f.into_canonical()
+                    .map_err(|err| {
+                        err.with_context(format!("Failed to canonicalize field {}", name))
+                    })
+                    .and_then(|c| c.into_arrow())
             })
             .collect::<VortexResult<Vec<_>>>()?;
 
@@ -212,77 +220,6 @@ fn struct_to_arrow(struct_array: StructArray) -> VortexResult<ArrayRef> {
     )?))
 }
 
-fn varbin_to_arrow(varbin_array: VarBinArray) -> VortexResult<ArrayRef> {
-    let offsets = varbin_array
-        .offsets()
-        .into_primitive()
-        .map_err(|err| err.with_context("Failed to canonicalize offsets"))?;
-    let offsets = match offsets.ptype() {
-        PType::I32 | PType::I64 => offsets,
-        PType::U64 => offsets.reinterpret_cast(PType::I64),
-        PType::U32 => offsets.reinterpret_cast(PType::I32),
-        // Unless it's u64, everything else can be converted into an i32.
-        _ => try_cast(offsets.to_array(), PType::I32.into())
-            .and_then(|a| a.into_primitive())
-            .map_err(|err| err.with_context("Failed to cast offsets to PrimitiveArray of i32"))?,
-    };
-    let nulls = varbin_array
-        .logical_validity()
-        .to_null_buffer()
-        .map_err(|err| err.with_context("Failed to get null buffer from logical validity"))?;
-
-    let data = varbin_array
-        .bytes()
-        .into_primitive()
-        .map_err(|err| err.with_context("Failed to canonicalize bytes"))?;
-    if data.ptype() != PType::U8 {
-        vortex_bail!("Expected bytes to be of type U8, got {}", data.ptype());
-    }
-    let data = data.buffer();
-
-    // Switch on Arrow DType.
-    Ok(match varbin_array.dtype() {
-        DType::Binary(_) => match offsets.ptype() {
-            PType::I32 => Arc::new(unsafe {
-                BinaryArray::new_unchecked(
-                    as_offset_buffer::<i32>(offsets),
-                    data.clone().into_arrow(),
-                    nulls,
-                )
-            }),
-            PType::I64 => Arc::new(unsafe {
-                LargeBinaryArray::new_unchecked(
-                    as_offset_buffer::<i64>(offsets),
-                    data.clone().into_arrow(),
-                    nulls,
-                )
-            }),
-            _ => vortex_bail!("Invalid offsets type {}", offsets.ptype()),
-        },
-        DType::Utf8(_) => match offsets.ptype() {
-            PType::I32 => Arc::new(unsafe {
-                StringArray::new_unchecked(
-                    as_offset_buffer::<i32>(offsets),
-                    data.clone().into_arrow(),
-                    nulls,
-                )
-            }),
-            PType::I64 => Arc::new(unsafe {
-                LargeStringArray::new_unchecked(
-                    as_offset_buffer::<i64>(offsets),
-                    data.clone().into_arrow(),
-                    nulls,
-                )
-            }),
-            _ => vortex_bail!("Invalid offsets type {}", offsets.ptype()),
-        },
-        _ => vortex_bail!(
-            "expected utf8 or binary instead of {}",
-            varbin_array.dtype()
-        ),
-    })
-}
-
 fn temporal_to_arrow(temporal_array: TemporalArray) -> VortexResult<ArrayRef> {
     macro_rules! extract_temporal_values {
         ($values:expr, $prim:ty) => {{
@@ -371,6 +308,25 @@ pub trait IntoCanonical {
     fn into_canonical(self) -> VortexResult<Canonical>;
 }
 
+/// Encoding VTable for canonicalizing an array.
+#[allow(clippy::wrong_self_convention)]
+pub trait IntoCanonicalVTable {
+    fn into_canonical(&self, array: ArrayData) -> VortexResult<Canonical>;
+}
+
+/// Implement the [IntoCanonicalVTable] for all encodings with arrays implementing [IntoCanonical].
+impl<E: Encoding> IntoCanonicalVTable for E
+where
+    E::Array: IntoCanonical,
+    E::Array: TryFrom<ArrayData, Error = VortexError>,
+{
+    fn into_canonical(&self, data: ArrayData) -> VortexResult<Canonical> {
+        let canonical = E::Array::try_from(data.clone())?.into_canonical()?;
+        canonical.inherit_statistics(data.statistics());
+        Ok(canonical)
+    }
+}
+
 /// Trait for types that can be converted from an owned type into an owned array variant.
 ///
 /// # Canonicalization
@@ -385,7 +341,7 @@ pub trait IntoArrayVariant {
 
     fn into_struct(self) -> VortexResult<StructArray>;
 
-    fn into_varbin(self) -> VortexResult<VarBinArray>;
+    fn into_varbinview(self) -> VortexResult<VarBinViewArray>;
 
     fn into_extension(self) -> VortexResult<ExtensionArray>;
 }
@@ -410,8 +366,8 @@ where
         self.into_canonical()?.into_struct()
     }
 
-    fn into_varbin(self) -> VortexResult<VarBinArray> {
-        self.into_canonical()?.into_varbin()
+    fn into_varbinview(self) -> VortexResult<VarBinViewArray> {
+        self.into_canonical()?.into_varbinview()
     }
 
     fn into_extension(self) -> VortexResult<ExtensionArray> {
@@ -423,26 +379,53 @@ where
 ///
 /// Canonicalizing an array requires potentially decompressing, so this requires a roundtrip through
 /// the array's internal codec.
-impl IntoCanonical for Array {
+impl IntoCanonical for ArrayData {
     fn into_canonical(self) -> VortexResult<Canonical> {
-        ArrayEncoding::canonicalize(self.encoding(), self)
+        log::debug!("Canonicalizing array with encoding {:?}", self.encoding());
+        self.encoding().into_canonical(self)
     }
 }
 
 /// This conversion is always "free" and should not touch underlying data. All it does is create an
 /// owned pointer to the underlying concrete array type.
 ///
-/// This combined with the above [IntoCanonical] impl for [Array] allows simple two-way conversions
+/// This combined with the above [IntoCanonical] impl for [ArrayData] allows simple two-way conversions
 /// between arbitrary Vortex encodings and canonical Arrow-compatible encodings.
-impl From<Canonical> for Array {
+impl From<Canonical> for ArrayData {
     fn from(value: Canonical) -> Self {
         match value {
-            Canonical::Null(a) => a.into(),
-            Canonical::Bool(a) => a.into(),
-            Canonical::Primitive(a) => a.into(),
-            Canonical::Struct(a) => a.into(),
-            Canonical::VarBin(a) => a.into(),
-            Canonical::Extension(a) => a.into(),
+            Canonical::Null(a) => a.into_array(),
+            Canonical::Bool(a) => a.into_array(),
+            Canonical::Primitive(a) => a.into_array(),
+            Canonical::Struct(a) => a.into_array(),
+            Canonical::VarBinView(a) => a.into_array(),
+            Canonical::Extension(a) => a.into_array(),
+        }
+    }
+}
+
+impl AsRef<ArrayData> for Canonical {
+    fn as_ref(&self) -> &ArrayData {
+        match self {
+            Canonical::Null(a) => a.as_ref(),
+            Canonical::Bool(a) => a.as_ref(),
+            Canonical::Primitive(a) => a.as_ref(),
+            Canonical::Struct(a) => a.as_ref(),
+            Canonical::VarBinView(a) => a.as_ref(),
+            Canonical::Extension(a) => a.as_ref(),
+        }
+    }
+}
+
+impl IntoArrayData for Canonical {
+    fn into_array(self) -> ArrayData {
+        match self {
+            Canonical::Null(a) => a.into_array(),
+            Canonical::Bool(a) => a.into_array(),
+            Canonical::Primitive(a) => a.into_array(),
+            Canonical::Struct(a) => a.into_array(),
+            Canonical::VarBinView(a) => a.into_array(),
+            Canonical::Extension(a) => a.into_array(),
         }
     }
 }
@@ -454,7 +437,7 @@ mod test {
     use arrow_array::cast::AsArray;
     use arrow_array::types::{Int32Type, Int64Type, UInt64Type};
     use arrow_array::{
-        Array, PrimitiveArray as ArrowPrimitiveArray, StringArray, StructArray as ArrowStructArray,
+        PrimitiveArray as ArrowPrimitiveArray, StringViewArray, StructArray as ArrowStructArray,
     };
     use arrow_buffer::NullBufferBuilder;
     use arrow_schema::{DataType, Field};
@@ -462,7 +445,7 @@ mod test {
     use crate::array::{PrimitiveArray, SparseArray, StructArray};
     use crate::arrow::FromArrowArray;
     use crate::validity::Validity;
-    use crate::{IntoArray, IntoCanonical};
+    use crate::{ArrayData, IntoArrayData, IntoCanonical};
 
     #[test]
     fn test_canonicalize_nested_struct() {
@@ -489,9 +472,11 @@ mod test {
                     .unwrap()
                     .into_array(),
                 )])
+                .unwrap()
                 .into_array(),
             ),
-        ]);
+        ])
+        .unwrap();
 
         let arrow_struct = nested_struct_array
             .into_canonical()
@@ -515,8 +500,7 @@ mod test {
             .as_any()
             .downcast_ref::<ArrowStructArray>()
             .cloned()
-            .unwrap()
-            .clone();
+            .unwrap();
 
         let inner_a = inner_struct
             .column(0)
@@ -536,7 +520,7 @@ mod test {
         nulls.append_n_non_nulls(4);
         nulls.append_null();
         nulls.append_non_null();
-        let names = Arc::new(StringArray::from_iter(vec![
+        let names = Arc::new(StringViewArray::from_iter(vec![
             Some("Joseph"),
             None,
             Some("Angela"),
@@ -555,7 +539,7 @@ mod test {
 
         let arrow_struct = ArrowStructArray::new(
             vec![
-                Arc::new(Field::new("name", DataType::Utf8, true)),
+                Arc::new(Field::new("name", DataType::Utf8View, true)),
                 Arc::new(Field::new("age", DataType::Int32, true)),
             ]
             .into(),
@@ -563,7 +547,7 @@ mod test {
             nulls.finish(),
         );
 
-        let vortex_struct = crate::Array::from_arrow(&arrow_struct, true);
+        let vortex_struct = ArrayData::from_arrow(&arrow_struct, true);
 
         assert_eq!(
             &arrow_struct,
