@@ -1,10 +1,11 @@
+use std::fmt::{Debug, Formatter};
 use std::ops::Deref;
 use std::str::Utf8Error;
 
 use crate::Buffer;
 
 /// A wrapper around a [`Buffer`] that guarantees that the buffer contains valid UTF-8.
-#[derive(Debug, Clone, PartialEq, Eq, PartialOrd)]
+#[derive(Clone, PartialEq, Eq, PartialOrd)]
 pub struct BufferString(Buffer);
 
 impl BufferString {
@@ -23,6 +24,14 @@ impl BufferString {
     }
 }
 
+impl Debug for BufferString {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("BufferString")
+            .field("string", &self.as_str())
+            .finish()
+    }
+}
+
 impl From<BufferString> for Buffer {
     fn from(value: BufferString) -> Self {
         value.0
@@ -57,3 +66,9 @@ impl AsRef<str> for BufferString {
         self.as_str()
     }
 }
+
+impl AsRef<[u8]> for BufferString {
+    fn as_ref(&self) -> &[u8] {
+        self.as_str().as_bytes()
+    }
+}
