@@ -8,11 +8,8 @@ use crate::{flatbuffers as fb, DType, StructDType};
 /// Convert name references in projection list into index references.
 ///
 /// This is mostly useful if you want to deduplicate multiple projections against serialized schema.
-pub fn resolve_field_references<'a, 'b: 'a>(
-    fb: fb::Struct_<'b>,
-    projection: &'a [Field],
-) -> impl Iterator<Item = VortexResult<usize>> + 'a {
-    projection.iter().map(move |field| match field {
+pub fn resolve_field<'a, 'b: 'a>(fb: fb::Struct_<'b>, field: &'a Field) -> VortexResult<usize> {
+    match field {
         Field::Name(n) => {
             let names = fb
                 .names()
@@ -23,11 +20,20 @@ pub fn resolve_field_references<'a, 'b: 'a>(
                 .ok_or_else(|| vortex_err!("Unknown field name {n}"))
         }
         Field::Index(i) => Ok(*i),
-    })
+    }
+}
+
+/// Deserialize single field out of a struct dtype and as a top level dtype
+pub fn extract_field(fb_dtype: fb::DType<'_>, field: &Field) -> VortexResult<DType> {
+    let fb_struct = fb_dtype
+        .type__as_struct_()
+        .ok_or_else(|| vortex_err!("The top-level type should be a struct"))?;
+    let (_, dtype) = read_field(fb_struct, resolve_field(fb_struct, field)?)?;
+    Ok(dtype)
 }
 
 /// Deserialize flatbuffer schema selecting only columns defined by projection
-pub fn deserialize_and_project(
+pub fn project_and_deserialize(
     fb_dtype: fb::DType<'_>,
     projection: &[Field],
 ) -> VortexResult<DType> {
@@ -36,12 +42,13 @@ pub fn deserialize_and_project(
         .ok_or_else(|| vortex_err!("The top-level type should be a struct"))?;
     let nullability = fb_struct.nullable().into();
 
-    let (names, dtypes): (Vec<Arc<str>>, Vec<DType>) =
-        resolve_field_references(fb_struct, projection)
-            .map(|idx| idx.and_then(|i| read_field(fb_struct, i)))
-            .collect::<VortexResult<Vec<_>>>()?
-            .into_iter()
-            .unzip();
+    let (names, dtypes): (Vec<Arc<str>>, Vec<DType>) = projection
+        .iter()
+        .map(|f| resolve_field(fb_struct, f))
+        .map(|idx| idx.and_then(|i| read_field(fb_struct, i)))
+        .collect::<VortexResult<Vec<_>>>()?
+        .into_iter()
+        .unzip();
 
     Ok(DType::Struct(
         StructDType::new(names.into(), dtypes),
