@@ -1,8 +1,8 @@
-use std::collections::HashSet;
-
-use vortex::array::TemporalArray;
-use vortex::encoding::EncodingRef;
-use vortex::{Array, ArrayDType, ArrayDef, IntoArray};
+use vortex_array::aliases::hash_set::HashSet;
+use vortex_array::array::TemporalArray;
+use vortex_array::encoding::EncodingRef;
+use vortex_array::stats::ArrayStatistics;
+use vortex_array::{ArrayDType, ArrayData, ArrayDef, IntoArrayData};
 use vortex_datetime_dtype::TemporalMetadata;
 use vortex_datetime_parts::{
     split_temporal, DateTimeParts, DateTimePartsArray, DateTimePartsEncoding, TemporalParts,
@@ -10,7 +10,7 @@ use vortex_datetime_parts::{
 use vortex_error::VortexResult;
 
 use crate::compressors::{CompressedArray, CompressionTree, EncodingCompressor};
-use crate::SamplingCompressor;
+use crate::{constants, SamplingCompressor};
 
 #[derive(Debug)]
 pub struct DateTimePartsCompressor;
@@ -20,8 +20,12 @@ impl EncodingCompressor for DateTimePartsCompressor {
         DateTimeParts::ID.as_ref()
     }
 
-    fn can_compress(&self, array: &Array) -> Option<&dyn EncodingCompressor> {
-        if let Ok(temporal_array) = TemporalArray::try_from(array) {
+    fn cost(&self) -> u8 {
+        constants::DATE_TIME_PARTS_COST
+    }
+
+    fn can_compress(&self, array: &ArrayData) -> Option<&dyn EncodingCompressor> {
+        if let Ok(temporal_array) = TemporalArray::try_from(array.clone()) {
             match temporal_array.temporal_metadata() {
                 // We only attempt to compress Timestamp arrays.
                 TemporalMetadata::Timestamp(..) => Some(self),
@@ -34,7 +38,7 @@ impl EncodingCompressor for DateTimePartsCompressor {
 
     fn compress<'a>(
         &'a self,
-        array: &Array,
+        array: &ArrayData,
         like: Option<CompressionTree<'a>>,
         ctx: SamplingCompressor<'a>,
     ) -> VortexResult<CompressedArray<'a>> {
@@ -42,7 +46,7 @@ impl EncodingCompressor for DateTimePartsCompressor {
             days,
             seconds,
             subseconds,
-        } = split_temporal(TemporalArray::try_from(array)?)?;
+        } = split_temporal(TemporalArray::try_from(array.clone())?)?;
 
         let days = ctx
             .named("days")
@@ -53,7 +57,7 @@ impl EncodingCompressor for DateTimePartsCompressor {
         let subsecond = ctx
             .named("subsecond")
             .compress(&subseconds, like.as_ref().and_then(|l| l.child(2)))?;
-        Ok(CompressedArray::new(
+        Ok(CompressedArray::compressed(
             DateTimePartsArray::try_new(
                 array.dtype().clone(),
                 days.array,
@@ -65,6 +69,7 @@ impl EncodingCompressor for DateTimePartsCompressor {
                 self,
                 vec![days.path, seconds.path, subsecond.path],
             )),
+            Some(array.statistics()),
         ))
     }
 
