@@ -1,13 +1,14 @@
-use std::collections::HashSet;
-
-use vortex::array::PrimitiveArray;
-use vortex::encoding::EncodingRef;
-use vortex::{Array, ArrayDef, IntoArray};
+use vortex_array::aliases::hash_set::HashSet;
+use vortex_array::array::PrimitiveArray;
+use vortex_array::encoding::EncodingRef;
+use vortex_array::stats::ArrayStatistics;
+use vortex_array::variants::PrimitiveArrayTrait;
+use vortex_array::{ArrayData, ArrayDef, IntoArrayData};
 use vortex_error::VortexResult;
 use vortex_fastlanes::{delta_compress, Delta, DeltaArray, DeltaEncoding};
 
 use crate::compressors::{CompressedArray, CompressionTree, EncodingCompressor};
-use crate::SamplingCompressor;
+use crate::{constants, SamplingCompressor};
 
 #[derive(Debug)]
 pub struct DeltaCompressor;
@@ -17,9 +18,13 @@ impl EncodingCompressor for DeltaCompressor {
         Delta::ID.as_ref()
     }
 
-    fn can_compress(&self, array: &Array) -> Option<&dyn EncodingCompressor> {
+    fn cost(&self) -> u8 {
+        constants::DELTA_COST
+    }
+
+    fn can_compress(&self, array: &ArrayData) -> Option<&dyn EncodingCompressor> {
         // Only support primitive arrays
-        let parray = PrimitiveArray::try_from(array).ok()?;
+        let parray = PrimitiveArray::try_from(array.clone()).ok()?;
 
         // Only supports ints
         if !parray.ptype().is_unsigned_int() {
@@ -31,11 +36,11 @@ impl EncodingCompressor for DeltaCompressor {
 
     fn compress<'a>(
         &'a self,
-        array: &Array,
+        array: &ArrayData,
         like: Option<CompressionTree<'a>>,
         ctx: SamplingCompressor<'a>,
     ) -> VortexResult<CompressedArray<'a>> {
-        let parray = PrimitiveArray::try_from(array)?;
+        let parray = PrimitiveArray::try_from(array.clone())?;
         let validity = ctx.compress_validity(parray.validity())?;
 
         // Compress the filled array
@@ -49,10 +54,11 @@ impl EncodingCompressor for DeltaCompressor {
             .named("deltas")
             .compress(deltas.as_ref(), like.as_ref().and_then(|l| l.child(1)))?;
 
-        Ok(CompressedArray::new(
+        Ok(CompressedArray::compressed(
             DeltaArray::try_from_delta_compress_parts(bases.array, deltas.array, validity)?
                 .into_array(),
             Some(CompressionTree::new(self, vec![bases.path, deltas.path])),
+            Some(array.statistics()),
         ))
     }
 
