@@ -1,19 +1,23 @@
 use std::any::Any;
-use std::collections::HashSet;
 use std::fmt::Debug;
 use std::sync::Arc;
 
 use fsst::Compressor;
-use vortex::array::{VarBin, VarBinArray, VarBinView};
-use vortex::encoding::EncodingRef;
-use vortex::{ArrayDType, ArrayDef, IntoArray};
+use vortex_array::aliases::hash_set::HashSet;
+use vortex_array::array::{VarBin, VarBinView};
+use vortex_array::encoding::EncodingRef;
+use vortex_array::stats::ArrayStatistics;
+use vortex_array::{ArrayDType, ArrayDef, IntoArrayData};
 use vortex_dtype::DType;
 use vortex_error::{vortex_bail, VortexResult};
 use vortex_fsst::{fsst_compress, fsst_train_compressor, FSSTArray, FSSTEncoding, FSST};
 
+use super::bitpacked::BITPACK_WITH_PATCHES;
 use super::delta::DeltaCompressor;
+use super::r#for::FoRCompressor;
+use super::varbin::VarBinCompressor;
 use super::{CompressedArray, CompressionTree, EncoderMetadata, EncodingCompressor};
-use crate::SamplingCompressor;
+use crate::{constants, SamplingCompressor};
 
 #[derive(Debug)]
 pub struct FSSTCompressor;
@@ -32,7 +36,11 @@ impl EncodingCompressor for FSSTCompressor {
         FSST::ID.as_ref()
     }
 
-    fn can_compress(&self, array: &vortex::Array) -> Option<&dyn EncodingCompressor> {
+    fn cost(&self) -> u8 {
+        constants::FSST_COST
+    }
+
+    fn can_compress(&self, array: &vortex_array::ArrayData) -> Option<&dyn EncodingCompressor> {
         // FSST arrays must have DType::Utf8.
         //
         // Note that while it can accept binary data, it is unlikely to perform well.
@@ -50,7 +58,7 @@ impl EncodingCompressor for FSSTCompressor {
 
     fn compress<'a>(
         &'a self,
-        array: &vortex::Array,
+        array: &vortex_array::ArrayData,
         // TODO(aduffy): reuse compressor from sample run if we have saved it off.
         like: Option<CompressionTree<'a>>,
         ctx: SamplingCompressor<'a>,
@@ -59,7 +67,7 @@ impl EncodingCompressor for FSSTCompressor {
         // between 2-3x depending on the text quality.
         //
         // It's not worth running a full compression step unless the array is large enough.
-        if array.nbytes() < 10 * FSST_SYMTAB_MAX_SIZE {
+        if array.nbytes() < 5 * FSST_SYMTAB_MAX_SIZE {
             return Ok(CompressedArray::uncompressed(array.clone()));
         }
 
@@ -83,49 +91,42 @@ impl EncodingCompressor for FSSTCompressor {
             )
         };
 
+        let codes = fsst_array.codes();
+        let compressed_codes = ctx
+            .auxiliary("fsst_codes")
+            .excluding(self)
+            .including_only(&[
+                &VarBinCompressor,
+                &DeltaCompressor,
+                &FoRCompressor,
+                &BITPACK_WITH_PATCHES,
+            ])
+            .compress(&codes, like.as_ref().and_then(|l| l.child(2)))?;
+
         // Compress the uncompressed_lengths array.
         let uncompressed_lengths = ctx
             .auxiliary("uncompressed_lengths")
             .excluding(self)
             .compress(
                 &fsst_array.uncompressed_lengths(),
-                like.as_ref().and_then(|l| l.child(0)),
+                like.as_ref().and_then(|l| l.child(3)),
             )?;
 
-        let codes_varbin = VarBinArray::try_from(fsst_array.codes())?;
-        let codes_varbin_dtype = codes_varbin.dtype().clone();
-
-        let codes_offsets_compressed = ctx
-            .named("fsst_codes_offsets")
-            .excluding(self)
-            .including(&DeltaCompressor)
-            .compress(
-                &codes_varbin.offsets(),
-                like.as_ref().and_then(|l| l.child(1)),
-            )?;
-
-        let codes = VarBinArray::try_new(
-            codes_offsets_compressed.array,
-            codes_varbin.bytes(),
-            codes_varbin_dtype,
-            codes_varbin.validity(),
-        )?
-        .into_array();
-
-        Ok(CompressedArray::new(
+        Ok(CompressedArray::compressed(
             FSSTArray::try_new(
                 fsst_array.dtype().clone(),
                 fsst_array.symbols(),
                 fsst_array.symbol_lengths(),
-                codes,
+                compressed_codes.array,
                 uncompressed_lengths.array,
             )?
             .into_array(),
             Some(CompressionTree::new_with_metadata(
                 self,
-                vec![uncompressed_lengths.path, codes_offsets_compressed.path],
+                vec![None, None, compressed_codes.path, uncompressed_lengths.path],
                 compressor,
             )),
+            Some(array.statistics()),
         ))
     }
 
