@@ -3,7 +3,7 @@ use vortex_dtype::{DType, Nullability};
 use vortex_error::{vortex_bail, vortex_err, VortexError, VortexResult};
 
 use crate::value::ScalarValue;
-use crate::Scalar;
+use crate::{InnerScalarValue, Scalar};
 
 pub struct BoolScalar<'a> {
     dtype: &'a DType,
@@ -29,13 +29,30 @@ impl<'a> BoolScalar<'a> {
             _ => vortex_bail!("Can't cast {} to bool", dtype),
         }
     }
+
+    pub fn invert(self) -> BoolScalar<'a> {
+        BoolScalar {
+            dtype: self.dtype,
+            value: self.value.map(|v| !v),
+        }
+    }
+
+    pub fn into_scalar(self) -> Scalar {
+        Scalar {
+            dtype: self.dtype.clone(),
+            value: self
+                .value
+                .map(|x| ScalarValue(InnerScalarValue::Bool(x)))
+                .unwrap_or_else(|| ScalarValue(InnerScalarValue::Null)),
+        }
+    }
 }
 
 impl Scalar {
     pub fn bool(value: bool, nullability: Nullability) -> Self {
         Self {
             dtype: DType::Bool(nullability),
-            value: ScalarValue::Bool(value),
+            value: ScalarValue(InnerScalarValue::Bool(value)),
         }
     }
 }
@@ -58,12 +75,35 @@ impl TryFrom<&Scalar> for bool {
     type Error = VortexError;
 
     fn try_from(value: &Scalar) -> VortexResult<Self> {
-        BoolScalar::try_from(value)?
-            .value()
+        <Option<bool>>::try_from(value)?
             .ok_or_else(|| vortex_err!("Can't extract present value from null scalar"))
     }
 }
 
+impl TryFrom<&Scalar> for Option<bool> {
+    type Error = VortexError;
+
+    fn try_from(value: &Scalar) -> VortexResult<Self> {
+        Ok(BoolScalar::try_from(value)?.value())
+    }
+}
+
+impl TryFrom<Scalar> for bool {
+    type Error = VortexError;
+
+    fn try_from(value: Scalar) -> VortexResult<Self> {
+        Self::try_from(&value)
+    }
+}
+
+impl TryFrom<Scalar> for Option<bool> {
+    type Error = VortexError;
+
+    fn try_from(value: Scalar) -> VortexResult<Self> {
+        Self::try_from(&value)
+    }
+}
+
 impl From<bool> for Scalar {
     fn from(value: bool) -> Self {
         Self {
@@ -73,19 +113,9 @@ impl From<bool> for Scalar {
     }
 }
 
-impl TryFrom<&ScalarValue> for bool {
-    type Error = VortexError;
-
-    fn try_from(value: &ScalarValue) -> VortexResult<Self> {
-        value
-            .as_bool()?
-            .ok_or_else(|| vortex_err!("Can't extract present value from null scalar"))
-    }
-}
-
 impl From<bool> for ScalarValue {
     fn from(value: bool) -> Self {
-        ScalarValue::Bool(value)
+        ScalarValue(InnerScalarValue::Bool(value))
     }
 }
 
