@@ -1,5 +1,7 @@
+use std::sync::Arc;
+
 use vortex_dtype::{DType, ExtDType};
-use vortex_error::{vortex_bail, VortexError, VortexResult};
+use vortex_error::{vortex_bail, vortex_panic, VortexError, VortexResult};
 
 use crate::value::ScalarValue;
 use crate::Scalar;
@@ -25,9 +27,14 @@ impl<'a> ExtScalar<'a> {
         self.dtype
     }
 
-    /// Returns the stored value of the extension scalar.
-    pub fn value(&self) -> &'a ScalarValue {
-        self.value
+    /// Returns the storage scalar of the extension scalar.
+    pub fn storage(&self) -> Scalar {
+        let storage_dtype = if let DType::Extension(ext_dtype) = self.dtype() {
+            ext_dtype.storage_dtype().clone()
+        } else {
+            vortex_panic!("Expected extension DType: {}", self.dtype());
+        };
+        Scalar::new(storage_dtype, self.value.clone())
     }
 
     pub fn cast(&self, _dtype: &DType) -> VortexResult<Scalar> {
@@ -44,10 +51,10 @@ impl<'a> TryFrom<&'a Scalar> for ExtScalar<'a> {
 }
 
 impl Scalar {
-    pub fn extension(ext_dtype: ExtDType, storage: Self) -> Self {
+    pub fn extension(ext_dtype: Arc<ExtDType>, value: Scalar) -> Self {
         Self {
-            dtype: DType::Extension(ext_dtype, storage.dtype().nullability()),
-            value: storage.value,
+            dtype: DType::Extension(ext_dtype),
+            value: value.value().clone(),
         }
     }
 }
