@@ -1,4 +1,6 @@
-use num_traits::NumCast;
+use std::any::type_name;
+
+use num_traits::{FromPrimitive, NumCast};
 use vortex_dtype::half::f16;
 use vortex_dtype::{match_each_native_ptype, DType, NativePType, Nullability, PType};
 use vortex_error::{
@@ -7,7 +9,7 @@ use vortex_error::{
 
 use crate::pvalue::PValue;
 use crate::value::ScalarValue;
-use crate::Scalar;
+use crate::{InnerScalarValue, Scalar};
 
 #[derive(Debug, Clone)]
 pub struct PrimitiveScalar<'a> {
@@ -60,9 +62,7 @@ impl<'a> PrimitiveScalar<'a> {
             T::PTYPE
         );
 
-        self.pvalue
-            .as_ref()
-            .map(|pv| T::try_from(*pv).vortex_unwrap())
+        self.pvalue.map(|pv| pv.as_primitive::<T>().vortex_unwrap())
     }
 
     pub fn cast(&self, dtype: &DType) -> VortexResult<Scalar> {
@@ -77,6 +77,79 @@ impl<'a> PrimitiveScalar<'a> {
             })
         })
     }
+
+    /// Attempt to extract the primitive value as the given type.
+    /// Fails on a bad cast.
+    pub fn as_<T: FromPrimitiveOrF16>(&self) -> VortexResult<Option<T>> {
+        match self.pvalue {
+            None => Ok(None),
+            Some(pv) => Ok(Some(match pv {
+                PValue::U8(v) => T::from_u8(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast u8 to {}", type_name::<T>())),
+                PValue::U16(v) => T::from_u16(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast u16 to {}", type_name::<T>())),
+                PValue::U32(v) => T::from_u32(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast u32 to {}", type_name::<T>())),
+                PValue::U64(v) => T::from_u64(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast u64 to {}", type_name::<T>())),
+                PValue::I8(v) => T::from_i8(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast i8 to {}", type_name::<T>())),
+                PValue::I16(v) => T::from_i16(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast i16 to {}", type_name::<T>())),
+                PValue::I32(v) => T::from_i32(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast i32 to {}", type_name::<T>())),
+                PValue::I64(v) => T::from_i64(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast i64 to {}", type_name::<T>())),
+                PValue::F16(v) => T::from_f16(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast f16 to {}", type_name::<T>())),
+                PValue::F32(v) => T::from_f32(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast f32 to {}", type_name::<T>())),
+                PValue::F64(v) => T::from_f64(v)
+                    .ok_or_else(|| vortex_err!("Failed to cast f64 to {}", type_name::<T>())),
+            }?)),
+        }
+    }
+}
+
+pub trait FromPrimitiveOrF16: FromPrimitive {
+    fn from_f16(v: f16) -> Option<Self>;
+}
+
+macro_rules! from_primitive_or_f16_for_non_floating_point {
+    ($T:ty) => {
+        impl FromPrimitiveOrF16 for $T {
+            fn from_f16(_: f16) -> Option<Self> {
+                None
+            }
+        }
+    };
+}
+
+from_primitive_or_f16_for_non_floating_point!(u8);
+from_primitive_or_f16_for_non_floating_point!(u16);
+from_primitive_or_f16_for_non_floating_point!(u32);
+from_primitive_or_f16_for_non_floating_point!(u64);
+from_primitive_or_f16_for_non_floating_point!(i8);
+from_primitive_or_f16_for_non_floating_point!(i16);
+from_primitive_or_f16_for_non_floating_point!(i32);
+from_primitive_or_f16_for_non_floating_point!(i64);
+
+impl FromPrimitiveOrF16 for f16 {
+    fn from_f16(v: f16) -> Option<Self> {
+        Some(v)
+    }
+}
+
+impl FromPrimitiveOrF16 for f32 {
+    fn from_f16(v: f16) -> Option<Self> {
+        Some(v.to_f32())
+    }
+}
+
+impl FromPrimitiveOrF16 for f64 {
+    fn from_f16(v: f16) -> Option<Self> {
+        Some(v.to_f64())
+    }
 }
 
 impl<'a> TryFrom<&'a Scalar> for PrimitiveScalar<'a> {
@@ -91,7 +164,7 @@ impl Scalar {
     pub fn primitive<T: NativePType + Into<PValue>>(value: T, nullability: Nullability) -> Self {
         Self {
             dtype: DType::Primitive(T::PTYPE, nullability),
-            value: ScalarValue::Primitive(value.into()),
+            value: ScalarValue(InnerScalarValue::Primitive(value.into())),
         }
     }
 
@@ -114,45 +187,26 @@ impl Scalar {
             primitive
                 .pvalue
                 .map(|p| p.reinterpret_cast(ptype))
-                .map(ScalarValue::Primitive)
-                .unwrap_or_else(|| ScalarValue::Null),
+                .map(|x| ScalarValue(InnerScalarValue::Primitive(x)))
+                .unwrap_or_else(|| ScalarValue(InnerScalarValue::Null)),
         )
     }
 
     pub fn zero<T: NativePType + Into<PValue>>(nullability: Nullability) -> Self {
         Self {
             dtype: DType::Primitive(T::PTYPE, nullability),
-            value: ScalarValue::Primitive(T::zero().into()),
+            value: ScalarValue(InnerScalarValue::Primitive(T::zero().into())),
         }
     }
 }
 
 macro_rules! primitive_scalar {
     ($T:ty) => {
-        impl From<$T> for Scalar {
-            fn from(value: $T) -> Self {
-                Scalar {
-                    dtype: DType::Primitive(<$T>::PTYPE, Nullability::NonNullable),
-                    value: value.into(),
-                }
-            }
-        }
-
-        impl From<Option<$T>> for Scalar {
-            fn from(value: Option<$T>) -> Self {
-                Scalar {
-                    dtype: DType::Primitive(<$T>::PTYPE, Nullability::Nullable),
-                    value: value.into(),
-                }
-            }
-        }
-
         impl TryFrom<&Scalar> for $T {
             type Error = VortexError;
 
             fn try_from(value: &Scalar) -> Result<Self, Self::Error> {
-                PrimitiveScalar::try_from(value)?
-                    .typed_value::<$T>()
+                <Option<$T>>::try_from(value)?
                     .ok_or_else(|| vortex_err!("Can't extract present value from null scalar"))
             }
         }
@@ -165,38 +219,30 @@ macro_rules! primitive_scalar {
             }
         }
 
-        impl From<$T> for ScalarValue {
+        impl TryFrom<&Scalar> for Option<$T> {
+            type Error = VortexError;
+
+            fn try_from(value: &Scalar) -> Result<Self, Self::Error> {
+                Ok(PrimitiveScalar::try_from(value)?.typed_value::<$T>())
+            }
+        }
+
+        impl TryFrom<Scalar> for Option<$T> {
+            type Error = VortexError;
+
+            fn try_from(value: Scalar) -> Result<Self, Self::Error> {
+                <Option<$T>>::try_from(&value)
+            }
+        }
+
+        impl From<$T> for Scalar {
             fn from(value: $T) -> Self {
-                ScalarValue::Primitive(value.into())
-            }
-        }
-
-        impl From<Option<$T>> for ScalarValue {
-            fn from(value: Option<$T>) -> Self {
-                value
-                    .map(|v| ScalarValue::Primitive(v.into()))
-                    .unwrap_or_else(|| ScalarValue::Null)
-            }
-        }
-
-        impl TryFrom<&ScalarValue> for $T {
-            type Error = VortexError;
-
-            fn try_from(value: &ScalarValue) -> Result<Self, Self::Error> {
-                match value {
-                    ScalarValue::Primitive(pvalue) => <$T>::try_from(*pvalue),
-                    _ => vortex_bail!("expected primitive"),
+                Scalar {
+                    dtype: DType::Primitive(<$T>::PTYPE, Nullability::NonNullable),
+                    value: ScalarValue(InnerScalarValue::Primitive(value.into())),
                 }
             }
         }
-
-        impl TryFrom<ScalarValue> for $T {
-            type Error = VortexError;
-
-            fn try_from(value: ScalarValue) -> Result<Self, Self::Error> {
-                <$T>::try_from(&value)
-            }
-        }
     };
 }
 
@@ -212,32 +258,21 @@ primitive_scalar!(f16);
 primitive_scalar!(f32);
 primitive_scalar!(f64);
 
-impl From<usize> for Scalar {
-    fn from(value: usize) -> Self {
-        Self::from(value as u64)
-    }
-}
-
 /// Read a scalar as usize. For usize only, we implicitly cast for better ergonomics.
 impl TryFrom<&Scalar> for usize {
     type Error = VortexError;
 
     fn try_from(value: &Scalar) -> Result<Self, Self::Error> {
-        value.value().try_into()
-    }
-}
-
-impl From<usize> for ScalarValue {
-    fn from(value: usize) -> Self {
-        ScalarValue::Primitive(PValue::U64(value as u64))
+        PrimitiveScalar::try_from(value)?
+            .as_::<u64>()?
+            .map(|v| v as Self)
+            .ok_or_else(|| vortex_err!("cannot convert Null to usize"))
     }
 }
 
 /// Read a scalar as usize. For usize only, we implicitly cast for better ergonomics.
-impl TryFrom<&ScalarValue> for usize {
-    type Error = VortexError;
-
-    fn try_from(value: &ScalarValue) -> Result<Self, Self::Error> {
-        u64::try_from(value).map(|v| v as Self)
+impl From<usize> for Scalar {
+    fn from(value: usize) -> Self {
+        Scalar::primitive(value as u64, Nullability::NonNullable)
     }
 }
