@@ -8,7 +8,7 @@ use vortex_dtype::half::f16;
 use vortex_dtype::{NativePType, PType};
 use vortex_error::{vortex_err, VortexError};
 
-#[derive(Debug, Clone, Copy, PartialEq)]
+#[derive(Debug, Clone, Copy)]
 pub enum PValue {
     U8(u8),
     U16(u16),
@@ -23,20 +23,39 @@ pub enum PValue {
     F64(f64),
 }
 
+impl PartialEq for PValue {
+    fn eq(&self, other: &Self) -> bool {
+        match (self, other) {
+            (Self::U8(s), Self::U8(o)) => s.is_eq(*o),
+            (Self::U16(s), Self::U16(o)) => s.is_eq(*o),
+            (Self::U32(s), Self::U32(o)) => s.is_eq(*o),
+            (Self::U64(s), Self::U64(o)) => s.is_eq(*o),
+            (Self::I8(s), Self::I8(o)) => s.is_eq(*o),
+            (Self::I16(s), Self::I16(o)) => s.is_eq(*o),
+            (Self::I32(s), Self::I32(o)) => s.is_eq(*o),
+            (Self::I64(s), Self::I64(o)) => s.is_eq(*o),
+            (Self::F16(s), Self::F16(o)) => s.is_eq(*o),
+            (Self::F32(s), Self::F32(o)) => s.is_eq(*o),
+            (Self::F64(s), Self::F64(o)) => s.is_eq(*o),
+            (..) => false,
+        }
+    }
+}
+
 impl PartialOrd for PValue {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         match (self, other) {
-            (Self::U8(s), Self::U8(o)) => Some(s.compare(*o)),
-            (Self::U16(s), Self::U16(o)) => Some(s.compare(*o)),
-            (Self::U32(s), Self::U32(o)) => Some(s.compare(*o)),
-            (Self::U64(s), Self::U64(o)) => Some(s.compare(*o)),
-            (Self::I8(s), Self::I8(o)) => Some(s.compare(*o)),
-            (Self::I16(s), Self::I16(o)) => Some(s.compare(*o)),
-            (Self::I32(s), Self::I32(o)) => Some(s.compare(*o)),
-            (Self::I64(s), Self::I64(o)) => Some(s.compare(*o)),
-            (Self::F16(s), Self::F16(o)) => Some(s.compare(*o)),
-            (Self::F32(s), Self::F32(o)) => Some(s.compare(*o)),
-            (Self::F64(s), Self::F64(o)) => Some(s.compare(*o)),
+            (Self::U8(s), Self::U8(o)) => Some(s.total_compare(*o)),
+            (Self::U16(s), Self::U16(o)) => Some(s.total_compare(*o)),
+            (Self::U32(s), Self::U32(o)) => Some(s.total_compare(*o)),
+            (Self::U64(s), Self::U64(o)) => Some(s.total_compare(*o)),
+            (Self::I8(s), Self::I8(o)) => Some(s.total_compare(*o)),
+            (Self::I16(s), Self::I16(o)) => Some(s.total_compare(*o)),
+            (Self::I32(s), Self::I32(o)) => Some(s.total_compare(*o)),
+            (Self::I64(s), Self::I64(o)) => Some(s.total_compare(*o)),
+            (Self::F16(s), Self::F16(o)) => Some(s.total_compare(*o)),
+            (Self::F32(s), Self::F32(o)) => Some(s.total_compare(*o)),
+            (Self::F64(s), Self::F64(o)) => Some(s.total_compare(*o)),
             (..) => None,
         }
     }
@@ -47,10 +66,18 @@ macro_rules! as_primitive {
         paste! {
             #[doc = "Access PValue as `" $T "`, returning `None` if conversion is unsuccessful"]
             pub fn [<as_ $T>](self) -> Option<$T> {
-                if let PValue::$PT(v) = self {
-                    Some(v)
-                } else {
-                    None
+                match self {
+                    PValue::U8(v) => <$T as NumCast>::from(v),
+                    PValue::U16(v) => <$T as NumCast>::from(v),
+                    PValue::U32(v) => <$T as NumCast>::from(v),
+                    PValue::U64(v) => <$T as NumCast>::from(v),
+                    PValue::I8(v) => <$T as NumCast>::from(v),
+                    PValue::I16(v) => <$T as NumCast>::from(v),
+                    PValue::I32(v) => <$T as NumCast>::from(v),
+                    PValue::I64(v) => <$T as NumCast>::from(v),
+                    PValue::F16(v) => <$T as NumCast>::from(v),
+                    PValue::F32(v) => <$T as NumCast>::from(v),
+                    PValue::F64(v) => <$T as NumCast>::from(v),
                 }
             }
         }
@@ -78,6 +105,13 @@ impl PValue {
         &self.ptype() == ptype
     }
 
+    #[inline]
+    pub fn as_primitive<T: NativePType + TryFrom<PValue, Error = VortexError>>(
+        &self,
+    ) -> Result<T, VortexError> {
+        T::try_from(*self)
+    }
+
     #[allow(clippy::transmute_int_to_float, clippy::transmute_float_to_int)]
     pub fn reinterpret_cast(&self, ptype: PType) -> Self {
         if ptype == self.ptype() {
@@ -183,6 +217,7 @@ int_pvalue!(u8, U8);
 int_pvalue!(u16, U16);
 int_pvalue!(u32, U32);
 int_pvalue!(u64, U64);
+int_pvalue!(usize, U64);
 int_pvalue!(i8, I8);
 int_pvalue!(i16, I16);
 int_pvalue!(i32, I32);
@@ -237,6 +272,24 @@ macro_rules! impl_pvalue {
     };
 }
 
+impl_pvalue!(u8, U8);
+impl_pvalue!(u16, U16);
+impl_pvalue!(u32, U32);
+impl_pvalue!(u64, U64);
+impl_pvalue!(i8, I8);
+impl_pvalue!(i16, I16);
+impl_pvalue!(i32, I32);
+impl_pvalue!(i64, I64);
+impl_pvalue!(f16, F16);
+impl_pvalue!(f32, F32);
+impl_pvalue!(f64, F64);
+
+impl From<usize> for PValue {
+    fn from(value: usize) -> PValue {
+        PValue::U64(value as u64)
+    }
+}
+
 impl Display for PValue {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -255,18 +308,6 @@ impl Display for PValue {
     }
 }
 
-impl_pvalue!(u8, U8);
-impl_pvalue!(u16, U16);
-impl_pvalue!(u32, U32);
-impl_pvalue!(u64, U64);
-impl_pvalue!(i8, I8);
-impl_pvalue!(i16, I16);
-impl_pvalue!(i32, I32);
-impl_pvalue!(i64, I64);
-impl_pvalue!(f16, F16);
-impl_pvalue!(f32, F32);
-impl_pvalue!(f64, F64);
-
 #[cfg(test)]
 mod test {
     use vortex_dtype::half::f16;
